/*
 * novand_com.c
 *
 * Nova buffer manager network communication program
 *
 *              version 1.03  08-OCT-1992 Masaharu NOMACHI   (KEK online)
 */

#include "novanet.h"
#include "nova.h"

exec_tx(msgsock,name,pri,req) int msgsock, pri, req; char *name; {
  struct NOVAD *nd;
  int size, rval,sig;
  char *data;
  char header[24];

  nd = nova_open(name,pri,req);
  if ((int)nd == -1) {
    err_open(msgsock,name);
    return;
  }
  send_ack(msgsock,nd);
  
  /*
   * transmit data (First 24 byte is "nova:XXXXXXXX:SSSSSSSS:\0"
   *                     XXXXXXXX is buffer size in HEX.
   *                     SSSSSSSS is buffer significance in HEX.
   */
  
  while(1) {
#if !defined(VXWORKS)
/*    alarm(10); */
    alarm(300);
#endif
    rval = nova_get_with_sig(nd,&data,&size,&sig);
    if (rval==0) {
      sprintf(header,"nova:%08X:%08X:\0",size,sig);
      rval = nnv_write(msgsock, header, sizeof(header));
      if (rval>0) rval = nnv_write(msgsock, data, size);
      nova_put(nd,data,0,0);
      if (rval <= 0) break;
    }
    else if (rval==1) {
      send_timeout(msgsock);
    }
    else {
      printf("** ERROR ** NOVA get %d\n",rval);
      rval = 1;
      break;
    }
  }
  nova_close(nd);
  if (rval==0) 
    printf("** Connection close **\n");
  else if (rval<0)
    printf("** ERROR ** writing on stream socket%X\n",errno);
  else
    printf("** ERROR ** \n",rval);
  return;
}

exec_rx(msgsock,name,pri,req) int msgsock, pri, req; char *name; {
  struct NOVAD *nd;
  int nova_size, size, sig, rval, i;
  char *data;
  char dumy[4];

  nd = nova_open(name,-pri,req);
  if ((int)nd == -1) {
    err_open(msgsock,name);
    return;
  }
  send_ack(msgsock,nd);

  /*
   * receive data (First 24 byte is "nova:XXXXXXXX:YYYYYYYY:\0"
   *                     XXXXXXXX is buffer size in HEX.
   *                     YYYYYYYY is buffer significance in HEX.
   */

  while(1) {
#if !defined(VXWORKS)
/*    alarm(10); */
    alarm(300);
#endif
    rval = nova_get(nd, &data, &nova_size);
    if (rval==0) {
      rval = nnv_read(msgsock, data, 24);
      if (rval ==0) break;
      if (rval < 0 ) printf("** ERROR ** reading socket %X\n",errno);
      rval = sscanf(data,"nova:%X:%X:",&size,&sig);
      if (size < nova_size) {
	rval = nnv_read(msgsock, data, size);
	if (rval ==0) break;
	if (rval < 0 ) printf("** ERROR ** reading socket %X\n",errno);
	nova_put(nd,data,size,sig);
      }
      else {
	rval = nnv_read(msgsock, data, nova_size);
	if (rval ==0) break;
	if (rval < 0 ) printf("** ERROR ** reading socket %X\n",errno);
	nova_put(nd,data,nova_size,sig);
	rval = nnv_skip(msgsock,size-nova_size);
	if (rval ==0) break;
	if (rval < 0 ) printf("** ERROR ** reading socket %X\n",errno);
      }
    }
    else if (rval==1) {
      send_timeout(msgsock);
    }
    else {
      printf("** ERROR ** NOVA get %d\n",rval);
      rval = 1;
      break;
    }
  }
  nova_close(nd);
  if (rval==0) 
    printf("** Connection close **\n");
  else if (rval<0)
    printf("** ERROR ** writing on stream socket%X\n",errno);
  else
    printf("** ERROR ** \n",rval);
  return;
}

err_open(msgsock,name) int msgsock; char *name; {
  printf("open error %s\n",name);
  return;
}

#ifndef VXWORKS
char ack[] = {"ack:"};
#else
extern char ack[];
#endif
char tmout[] ={"timeout:                "};

send_timeout(msgsock) int msgsock; {
  return nnv_write(msgsock,tmout,24);
}

send_ack(msgsock,nd) int msgsock; struct NOVAD *nd; {
  char temp[32];
  sprintf(temp,"ack:%08X:",nd->max_length);
  return nnv_write(msgsock,temp,strlen(temp));
}

wait_ack(msgsock) int msgsock; {
  char buf[32];
  int rval;
  rval = nnv_read(msgsock,buf,sizeof(buf));
  return rval;
}
