/********************************************************************
 * k2917.h   Kinetic K2917 registers                                *
 *                                                                  *
 *   24-MAR-1992 Y.Takeuchi                                         *
 *                                                                  *
 * 13-JUL-1992 Y.Takeuchi    modify K2917_ENABLE,DISABLE            *
 * 30-NOV-1992 Y.Takeuchi    modify                                 *
 *  5-MAR-1993 Y.Takeuchi    modify for unified UNIX camlib         *
 ********************************************************************/

/***** DMA *****/
#define DMA_DONE        0x8000
#define DMA_ERR         0x1000
#define DMA_ACT         0x0800
#define DOCR_INIT       0x8010
#define DMA_READ        0x0080
#define DMA_WRITE       0x0000
#define DMA_START       0x0080
#define DMA_ABORT       0x0010
#define DMA_INT_ENABLE  0x0008
#define DMA_RESET       0x0000  

/***** Interrupt *****/
#define INTR_INIT       0x0000
#define TEST_FLAG       0x0080
#define FLAG_AUTO_CLEAR 0x0040
#define INT_ENABLE      0x0010
#define INT_AUTO_CLEAR  0x0008
#define IRQ1            0x0001
#define IRQ2            0x0002
#define IRQ3            0x0003
#define IRQ4            0x0004
#define IRQ5            0x0005
#define IRQ6            0x0006
#define IRQ7            0x0007

/***** Main register *****/
#define AMR_INIT        0x007D
#define CMA_INIT        0x0000

#define AD              0x0001
#define INLINE          0x0060
#define BLOCK           0x0020
#define BIT24           0x0000
#define BIT16           0x0002
#define QSTOP           0x0000
#define QIGNO           0x0008
#define QREPE           0x0010
#define QSCAN           0x0018
#define HALT            0x0080

#define ERR             0x8000
#define ABT             0x4000
#define TMO             0x2000
#define RST             0x1000
#define LAM             0x0200
#define RDY             0x0100
#define DONE            0x0080
#define DMA             0x0040
#define WRITE           0x0020
#define NOX             0x0004
#define NOQ             0x0002
#define GO              0x0001

struct K_REG {
    unsigned short cser;
    unsigned short dummy02;
    unsigned short docr;
    unsigned short sccr;
    unsigned short dummy05;
    unsigned short mtc;
    unsigned short machi;
    unsigned short maclo;

    unsigned short dummy11;           /* 0x10 - 0x1F */
    unsigned short dummy12;
    unsigned short dummy13;
    unsigned short dummy14;
    unsigned short dummy15;
    unsigned short dummy16;
    unsigned short dummy17;
    unsigned short dummy18;

    unsigned short dummy21;           /* 0x20 - 0x2F */
    unsigned short dummy22;
    unsigned short dummy23;
    unsigned short dummy24;
    unsigned short dummy25;
    unsigned short dummy26;
    unsigned short dummy27;
    unsigned short dummy28;

    unsigned short dummy31;           /* 0x30 - 0x3F */
    unsigned short dummy32;
    unsigned short dummy33;
    unsigned short dummy34;
    unsigned short dummy35;
    unsigned short dummy36;
    unsigned short dummy37;
    unsigned short dummy38;

    unsigned short lamc;
    unsigned short donc;
    unsigned short empc;
    unsigned short aboc;
    unsigned short lamv;
    unsigned short donv;
    unsigned short empv;
    unsigned short abov;

    unsigned short dummy51;           /* 0x50 - 0x5F */
    unsigned short dummy52;
    unsigned short dummy53;
    unsigned short dummy54;
    unsigned short dummy55;
    unsigned short dummy56;
    unsigned short dummy57;
    unsigned short dummy58;

    unsigned short amr;
    unsigned short cmr;
    unsigned short cma;
    unsigned short cwc;
    unsigned short srr;
    unsigned short dlr;
    unsigned short dhr;
    unsigned short csr;
};

struct klist {
    char name[9];
    unsigned short cma;
    unsigned short wc;
};
