 
 
/*****************************************************************************
**
*
*
*
*-----------------------------------------------------------------------------
--
*
* Module:	
*
*-----------------------------------------------------------------------------
--
*
* Description:	This file describes contains constant and type
*		definitions for all of the routines to interface
*		to an exabyte tape drive. The routines are in Exabyte.c
*
*-----------------------------------------------------------------------------
--
*
* $Log$
*
******************************************************************************
*/
 
/************************************
*          Module Includes          *
************************************/
 
 
/************************************
*           Local Defines           *
************************************/
 
 
/************************************
*          Local Functions          *
************************************/
 
 
/************************************
*          Local Variables          *
************************************/
 
/**********************************************************/
/*     Constant Definitions for the Exabyte Module        */
/**********************************************************/
 
/* Exabyte Status Codes */
 
#define EXB_NO_ERROR		0x00
#define EXB_BUSY		0x01	/* command rejected, device busy */
#define EXB_NOT_READY		0x02
#define EXB_MEDIUM_ERROR	0x03
#define EXB_HARDWARE_ERROR	0x04
#define EXB_ILLEGAL_REQUEST	0x05
#define EXB_UNIT_ATTENTION	0x06
#define EXB_DATA_PROTECT	0x07
#define EXB_BLANK_CHECK		0x08
#define EXB_EXABYTE_UNIQUE	0x09
#define EXB_ABORTED_COMMAND	0x0B
#define EXB_VOLUME_OVERFLOW	0x0D
#define EXB_NO_SENSE		0x0f
 
/* Exabyte Control Function Codes */
 
#define EXB_TEST_READY		0x1
#define EXB_REWIND		0x2
#define EXB_SPACE		0x3
#define EXB_LOAD		0x4
#define EXB_UNLOAD		0x5
#define EXB_WRITE_FILEMARK	0x6
 
/* Motion and Reconnect thresholds for exbModeSelect */
 
#define FORMAT_8500	0x00
#define FORMAT_8200	0x14
 
#define THRESHOLD_16K	0x10
#define THRESHOLD_32K	0x20
#define THRESHOLD_48K	0x30
 
typedef struct
{
	unsigned char device_code,
	              device_qualifier,
		      versions,
		      unused_1,
		      additional_length,
		      vendor_unique,
		      unused_2,
		      unused_3,
		      vi_1,
		      vi_2,
		      vi_3,
		      vi_4,
		      vi_5,
		      vi_6,
		      vi_7,
		      vi_8,
		      pi_1,
		      pi_2,
		      pi_3,
		      pi_4,
		      pi_5,
		      pi_6,
		      pi_7,
		      pi_8;
}EXB_INQUIRY_DATA;
 
typedef struct
{
	unsigned	valid			:1,	/* indicates bytes 3 
thru 6 are valid */
			errorClass		:3, 	/* always 7 */
			errorCode		:4;	/* always 0 */
	unsigned char	segNum;				/* zero unless check 
is result of copy*/
	unsigned	filemark		:1, 	/* read a filemark */
			endOfMedia		:1,	/* Logical EOT or BOT 
*/
			ili			:1,	/* incorrect logical 
blk length req. */
			reserved		:1,
			senseKey		:4;	/* sense key gives 
error indication */
	unsigned char	infoBytes[4];	
	unsigned char 	adtlSenseLength,		/* 0e except for copy 
command */
			reserved1,
			reserved2,
			reserved3,
			reserved4,
			adtlSenseCode,			/* 04 when not ready 
*/
			adtlSenseQual,			/* sense Qualifier */
			reserved5,
			reserved6,
			errorCountHb,			/* high byte of error 
count */
			errorCountMB,			/* middle byte of 
error count */
			errorCountLb;			/* low byte of error 
count */
	unsigned	powerFail	 	:1,
			busParityError		:1,
			bufferParityError 	:1,
			mediaError		:1,
			errorCountOverflow	:1,
			tapeMotionError		:1,
			tapeNotPresent		:1,
			bot			:1,	/* beginning of tape 
*/
			transferAbort		:1,
			tapeMarkDetect		:1,
			writeProtect		:1,
			filemarkError		:1,
			underrunError		:1,
			mediaWriteError		:1,
			servoError		:1,
			formatterError		:1,
			futureStuff		:5,
			peot			:1,
			spliceError1		:1,
			spliceError2		:1;
	unsigned char	reserved7,
			remainTapeHb,			/* high byte of 
remaining tape */
			remainTapeMb,			/* middle byte of 
remaining tape */
			remainTapeLb;			/* low byte of 
remaining tape */
}EXB_SENSE_DATA;
 
/**********************************************************/
/*      Type Definitions for the Exabyte Functions        */
/**********************************************************/
 
 
typedef struct
{
	unsigned	senseDataValid		:1, 	/* indicates if bits 
1-7 are valid */
			tapeNotPresent		:1,
			writeProtect		:1,
			endOfMedia		:1,	/* Logical EOT or BOT 
*/
			bot			:1,	/* beginning of tape 
*/
			peot 			:1,
			filemark		:1,	/* read a filemark */
			ili			:1,	/* incorrect logical 
blk length req. */
			reserved		:20,
			errorCode		:4;	/* error code is 
always valid */
}EXB_STATUS;
 
/*****************************************************************************
**
*	exbInit()
*
*	Description:	Initializes and installs SCSI driver
*
*	Returns:	OK or ERROR
*
******************************************************************************
*/
extern	STATUS		exbInit(void);
 
 
/*****************************************************************************
**
*	exbOpen()
*
*	Description:	Creates the tape drive device, waits for it to be
*			ready and selects the mode
*
*	Returns:	device or ERROR
*
******************************************************************************
*/
extern 	SCSI_PHYS_DEV*	exbOpen(int scsi_bus_id);
 
 
/*****************************************************************************
**
*	exbClose()
*
*	Description:	Deletes the tape drive device.
*
*	Returns:	device or ERROR
*
******************************************************************************
*/
void exbClose(SCSI_PHYS_DEV * this_device);
 
 
/*****************************************************************************
**
*	exbModeSelect()
*
*	Description:	Selects the Mode for the tape drive
*
*	Returns:	EXB_STATUS
*
******************************************************************************
*/
extern	EXB_STATUS 	exbModeSelect(SCSI_PHYS_DEV* scsi_phys_dev, int 
densityCode,
int motionThreshold, int reconnectThreshold);
 
 
/*****************************************************************************
**
*	exbInquiry()
*
*	Description:	Inquires information from the tape drive
*
*	Returns:	EXB_STATUS
*
******************************************************************************
*/
extern	EXB_STATUS	exbInquiry(SCSI_PHYS_DEV*  scsi_phys_dev,
                            EXB_INQUIRY_DATA  *your_inquiry_buffer_ptr);
 
 
/*****************************************************************************
**
*	exbRequestSense()
*
*	Description:	Reads sense data for the tape drive
*
*	Returns:	EXB_STATUS
*
******************************************************************************
*/
extern	EXB_STATUS	exbRequestSense(SCSI_PHYS_DEV*  scsi_phys_dev,
                            EXB_SENSE_DATA  *your_sense_buffer_ptr);
 
 
/*****************************************************************************
**
*	exbRead()
*
*	Description:	Reads data from the tape
*
*	Returns:	EXB_STATUS
*			*byte_count_ptr = actual number of bytes read
*
******************************************************************************
*/
extern	EXB_STATUS	exbRead(SCSI_PHYS_DEV *scsi_phys_dev,
                    		void *buffer,
                    		int *byte_count_ptr);
 
 
/*****************************************************************************
**
*	exbWrite()
*
*	Description:	Writes data to the tape
*
*	Returns:	EXB_STATUS
*			*byte_count_ptr = actual number of bytes written
*
******************************************************************************
*/
extern	EXB_STATUS	exbWrite(SCSI_PHYS_DEV *scsi_phys_dev,
                                void *buffer,
                                int *byte_count_ptr);
 
extern	EXB_STATUS 	exbTestReady(SCSI_PHYS_DEV* scsi_phys_dev);
 
extern	EXB_STATUS 	exbRewind(SCSI_PHYS_DEV* scsi_phys_dev);
 
extern	EXB_STATUS 	exbSpace(SCSI_PHYS_DEV*    scsi_phys_dev,
                        	int             blks,
                        	int             *count_ptr);
 
extern	EXB_STATUS	exbLoadTape(SCSI_PHYS_DEV* scsi_phys_dev);
 
extern	EXB_STATUS	exbUnloadTape(SCSI_PHYS_DEV* scsi_phys_dev);
 
extern	EXB_STATUS 	exbWriteFilemark(SCSI_PHYS_DEV* scsi_phys_dev);
 
extern	EXB_STATUS	exbControl(SCSI_PHYS_DEV* scsi_phys_dev,
				int cntl_code,
				int blks,
				int *count_ptr);
 
 
extern void     exbStatusShow (EXB_STATUS status);
 
