/*---------------------------------------------------------------*/
/*                                                               */
/*      example.c                                                */
/*      Donald BRENNER                                           */
/*      Superconducting Super Collider Laboratory                */
/*      December 10, 1992                                        */
/*       Mods by R. Ball, 4/2/93                                 */
/*                                                               */
/*       This is supposed to be used as an example program       */
/*  to help a user understand how to create                      */
/*  a process which receives buffers from NOVA and then looks at */
/*  the events in the buffer.                                    */
/*                                                               */
/*---------------------------------------------------------------*/

#include <stdio.h>
#include "template.h"

   /*----------------------------------------------*/
   /*                                              */
   /* The header file where the user declares      */
   /* the variables and commands for the process.  */
   /*                                              */
   /*----------------------------------------------*/

#include "example.h"

#include "nova.h"
#define DEFAULT_NAME     "/tmp/nova"
#define NOVA_HOSTNAME_MAX  32    /*  extracted from novanet.h   */

   /*----------------------------------------------*/
   /*                                              */
   /*  DEFAULT_NAME is the default name for the    */
   /*    nova socket to be used.                   */
   /*  DEFAULT_PRI is the priority for nova        */
   /*  A low number means that nova might not send */
   /*  all the buffers to that process.            */
   /*                                              */
   /*----------------------------------------------*/

#define DEFAULT_PRI      20 

#define DEFAULT_SLEEP    1

#define  NTIMEOUT       0

int Process_Event();

   /*----------------------------------------------*/
   /*                                              */
   /*  Global variables for nova                   */
   /*                                              */
   /*----------------------------------------------*/

struct   NOVAD *nd;
int *data, length;





main(argc, argv)
int argc;
char * argv[];
{
   char name[NV_MAXNAME + 1];
   char sec_name[NV_MAXNAME + NOVA_HOSTNAME_MAX + 1];

   short s;

/*----------------------------------------------*/
/*                                              */
/*  Call to Template_Init which is in cmdlib.c  */
/*                                              */
/*----------------------------------------------*/ 
    
   if(Template_Init(get_name_from_args(argc,argv)) < 0){
     printf("Process of name %s already exists\n",
            get_name_from_args(argc,argv));
     printf("Please use -n to set a different name\n");
     exit(0);
   }

/*--------------------------------------------------*/
/*                                                  */
/*  Declare the user-level variable "event_sink"    */
/*  to be critical.  Once it is declared, pick up   */
/*  the critical value from the status path and     */
/*  store it locally.                               */
/*                                                  */
/*--------------------------------------------------*/

   Crit_Var("event_sink");
   read_value_from_sp(ownpname,"event_sink",&s,event_sink);

/*--------------------------------------------------*/
/*                                                  */
/*  This is how you open nova so that nova will     */ 
/*  send data buffers to the process.               */
/*  Access is through the UNIDAQ intermediary       */
/*  routines so that a choice can be made between   */
/*  network access (-m machine command-line option) */
/*  and access on the local machine (no such        */
/*  option).                                        */
/*  The priority of NOVA access may also be         */
/*  modified by -p prio command-line option.        */
/*  If the name of the nova socket is to be         */
/*  different than the default above, it should     */
/*  be set using environmental variables            */
/*  LOCAL_NOVA or REMOTE_NOVA.                      */
/*                                                  */
/*--------------------------------------------------*/

   UNI_nova_parms(DEFAULT_NAME, DEFAULT_PRI, name, sec_name);
   if ((int)(nd = (struct NOVAD *)UNI_nova_open(name, nova_prio, 1)) 
                == -1) {
       printf("Error in opening nova...\n");
       Send_Message ( ownpname, "EXIT" );
   }
   
/*----------------------------------------------*/
/*                                              */
/*  Call to Template_Main with pointers to      */
/*  functions which the user wishes called      */
/*  during each event.(Aslo in cmdlib.c)        */
/*  To see an example of a Process_Event see    */ 
/*  below.                                      */
/*                                              */
/*----------------------------------------------*/
   
   Template_Main((int (*)())Process_Event, NULL, NULL);
}


   /*----------------------------------------------*/
   /*                                              */
   /*   The Process_Event function..............   */
   /*                                              */
   /*----------------------------------------------*/
   
int Process_Event()
{
   int status;
   int pos;
  
/*-----------------------------------------------*/
/*                                               */
/* Example of how to get a buffer.               */
/* The following variables are declared as       */
/* global variables: nd, data, length.           */
/* Status is used to see if an actual buffer was */
/* returned.                                     */
/*                                               */
/*-----------------------------------------------*/
          
   status = UNI_nova_get(nd, &data, &length);

/*-----------------------------------------------*/
/*                                               */
/*  If status is not equal zero then the process */
/*  timed-out and no buffer was received.        */
/*                                               */
/*-----------------------------------------------*/
                
   if (status != 0) return (FALSE);
   
/*----------------------------------------------*/
/*                                              */
/*  This is where the user can preform some     */
/*  action on the buffer full of events.        */
/*                                              */
/*  ONE BUFFER CAN PACK SEVERAL EVENTS........  */
/*                                              */
/*  The first element of a buffer contains the  */
/*  length of that event.                       */
/*  pos = data[0]   = length of first event     */
/*  pos = data[pos]+pos = length of the         */ 
/*         next event                           */
/*                    .                         */
/*                    .                         */
/*  pos = data[pos]+pos = length of another     */
/*          event                               */
/*                                              */
/*  IF data[pos] = zero THEN the buffer contains*/
/*  no more events and should be passed back    */
/*  to nova.                                    */   
/*                                              */   
/*----------------------------------------------*/
   
   printf("NEW BUFFER RECEIVED..........\n");
   pos = 0;
   while (pos*4 < length)  {
     if (data[pos] == 0) break;
     printf("An event.....................\n",pos);
     pos = data[pos] + pos;
   }
   printf("NO MORE EVENTS IN BUFFER.....\n",pos);
   
/*----------------------------------------------*/
/*                                              */
/*  This is how a process passes back a buffer  */
/*  to nova so that nova can send the buffer to */
/*  another process                             */
/*                                              */
/*----------------------------------------------*/
   
   UNI_nova_put(nd,  data, length, 0); 
   
   return (TRUE);
}

   /*-------------------------------------------------*/
   /*                                                 */
   /*  This is an example of a Test command. See the  */
   /*  example.h header file for declarations         */
   /*  of the command and funcion                     */
   /*                                                 */
   /*-------------------------------------------------*/

int Test(header, arg_string)
char *header, *arg_string;
{

/*--------------------------------------------------*/
/*                                                  */
/*  This is where the user would add code to        */
/*  perform some action when the command is         */
/*  given.                                          */
/*                                                  */
/*     Arguments to the command are contained       */
/*     in 'arg_string' and can be separated         */
/*     using the Parse subroutine, eg               */
/*  char first_word[80], rest_of_string[80];        */
/*  Parse(arg_string, first_word, rest_of_string);  */
/*                                                  */
/*--------------------------------------------------*/
   
    printf("INSIDE the Test command.........\n");
    
    return TRUE;
}
   
   /*----------------------------------------------*/
   /*                                              */
   /*  An example of an exit command               */
   /*                                              */
   /*----------------------------------------------*/
   
int ExitR(header, arg_string)
char *header, *arg_string;
{

/*----------------------------------------------*/
/*                                              */
/*  The exit command should always close the    */
/*  nova connection.                            */
/*                                              */
/*----------------------------------------------*/
   
   UNI_nova_close(nd);
   
/*----------------------------------------------*/
/*                                              */
/*  Return with the template's exit so that     */
/*  message queues and shared memory space      */
/*  are deleted.                                */
/*                                              */
/*----------------------------------------------*/
      
   return (Exit(ownpname, " "));
}

