/* ///////////////////////////////////////////////////////////////////
//
// file:    gui_lib.C
// author:  C. Timmermans
// version: 2.0
// date:    3/09/93
/////////////////////////////////////////////////////////////////// */

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#if defined(HP) || defined(SGI)
#include <X11/Xlibint.h>
#endif
#include <X11/keysym.h>
#include <X11/Xatom.h>
#include<stdio.h>
/* These are used as arguments to nearly every Xlib routine, so it saves 
 * routine arguments to declare them global.  If there were 
 * additional source files, they would be declared extern there. */
Display *display;
int screen;
extern int errno;

struct win_pos
{
  Window win;
  u_int x,y,width,height,border;
  int hide;
};

struct win_pos Xrtpos[50];
int Xrtwin = 0;

int gui_init(display_name)
char *display_name;
/* ///////////////////////////////////////////////////////////////
// connects to the server, opens the display and gets the screen
////////////////////////////////////////////////////////////// */
{
	if ( (display=XOpenDisplay(display_name)) == NULL )
	{
		printf("Display: cannot connect to X server %s\n", 
	                XDisplayName(display_name));
		return(-1);
	}
	/* get screen */
	screen = DefaultScreen(display);
#ifdef _DEBUG
	printf("Opening display %d\n",(int)display);
#endif
	return(0);

}

void gui_window(win,gc,rt_win,width,height,x,y,border,msk,name)
Window *win,*rt_win;
GC *gc;
u_int width,height,x,y,border;
int msk;
char *name;
/* //////////////////////////////////////////////////////////////////////////
// Creates a very simple window named "name". The width and height of this
// window are always used. The x and y position are merely hints. It depends
// on the server if these are used or not. The value of msk determines
// to which actions the window responds.
////////////////////////////////////////////////////////////////////////// */
{
	XSizeHints hints;
	XGCValues values;
	XFontStruct *font_info;
	unsigned long evt_mask;
	char **argv;

/****      Create the window    ****/
        if(rt_win == 0) {
  	  *win = XCreateSimpleWindow(display, RootWindow(display,screen), x, y,
              width, height, border, BlackPixel(display,screen), 
              WhitePixel(display,screen));
	  Xrtpos[Xrtwin].win = *win;
	  Xrtpos[Xrtwin].hide = 1;
	  Xrtpos[Xrtwin].x = x;
	  Xrtpos[Xrtwin].y = y;
	  Xrtpos[Xrtwin].width = width;
	  Xrtpos[Xrtwin].height = height;
	  Xrtpos[Xrtwin++].border = border;
	}
	else
  	  *win = XCreateSimpleWindow(display, *rt_win, x, y,
              width, height, border, BlackPixel(display,screen), 
              WhitePixel(display,screen));

/****      set the hints        ****/
	hints.x = x;
	hints.y = y;
	hints.width = width;
	hints.height = height;
	hints.min_width = width;
	hints.max_width = width;
	hints.min_height = height;
	hints.max_height = height;
	hints.flags = PPosition | PSize | PMinSize | PMaxSize;
	XSetStandardProperties(display,*win,name,name,None,argv,0,&hints);

	evt_mask = ExposureMask | StructureNotifyMask;
	if((msk & 1) == 1) evt_mask = evt_mask | KeyPressMask;
	if((msk & 2) == 2) 
	  evt_mask = evt_mask | ButtonPressMask | ButtonReleaseMask;
	XSelectInput(display, *win, evt_mask);

/* And now the graphics content*/
	*gc = XCreateGC(display,*win,0,&values);
	XSetForeground(display,*gc,BlackPixel(display,screen));
}

void gui_text(win,gc,x,y,txt)
Window *win;
GC *gc;
int x,y;
char *txt;
/* /////////////////////////////////////////////////////////////
// draws a string into window "win" at position (x,y)
//////////////////////////////////////////////////////////// */
{
	XDrawString(display,*win,*gc,x,y,txt,strlen(txt));
}

void gui_display(win)
Window *win;
/* //////////////////////////////////////////////////////////////
// makes sure the window is visible
////////////////////////////////////////////////////////////// */
{
  int i;
  XWindowChanges xwc;

  XMapWindow(display,*win);
  for(i = 0;i<Xrtwin;i++){
    if(*win == Xrtpos[i].win) {
      Xrtpos[i].hide = 0;
      xwc.x = Xrtpos[i].x;
      xwc.y = Xrtpos[i].y;
      xwc.width = Xrtpos[i].width;
      xwc.height = Xrtpos[i].height;
      xwc.border_width = Xrtpos[i].border;
      XConfigureWindow(display,*win,
		       CWX | CWY | CWWidth | CWHeight | CWBorderWidth,
		       &xwc);
      return;
    }
  }
}

void gui_hide(win)
Window *win;
/* //////////////////////////////////////////////////////////////
// makes the window invisible
////////////////////////////////////////////////////////////// */
{
  int i;

  XUnmapWindow(display,*win);
  for(i = 0;i<Xrtwin;i++){
    if(*win == Xrtpos[i].win) {
      Xrtpos[i].hide = 1;
    }
  }
}

int gui_string(report,win,gc,string,x,y,MIN_CHAR,MAX_CHAR)
XEvent report;
Window *win;
GC *gc;
char *string;
int x,y,MIN_CHAR,MAX_CHAR;
/* //////////////////////////////////////////////////////////////////////
// handles character inputs in a window. It makes sure that the size of the
// character string is between MIN_CHAR and MAX_CHAR. (x,y) is the position
// of the string. The return value is 1 if a return is given; otherwise
// the return value is 0
////////////////////////////////////////////////////////////////////// */
{
	char buffer[100];
	int bufsize=100;
	KeySym keysym;
	XComposeStatus compose;
	int length;

	/* clear buffer so multi-character strings
	 * are properly supported, even though
	 * they can only be deleted on char at a
	 * time */
	buffer[0] = NULL;
	buffer[1] = NULL;

	XLookupString(&(report.xkey), buffer, bufsize, &keysym, &compose);
	/* now need to do the right thing with
	 * every keysym possibility, as minimum: */
	if ((keysym == XK_Return) || (keysym == XK_KP_Enter) || (keysym == XK_Linefeed)) {
	     XClearWindow(display, *win); 
	     return(1);
	}
	else if (((keysym >= XK_KP_Space) && (keysym <= XK_KP_9))
	  || ((keysym >= XK_space) && (keysym <= XK_asciitilde)))
 		{
		if (strlen(string) >= MAX_CHAR)
			XBell(display, 100);
		else 
			strcat(string, buffer);
		}
	else if ((keysym >= XK_Shift_L) && (keysym <= XK_Hyper_R))
		;/* do nothing because its a modifier key */
	else if ((keysym >= XK_F1) && (keysym <= XK_F35)){
		if (buffer == NULL) ;
		else if (strlen(string) >= MAX_CHAR)
			XBell(display, 100);
		else 
			strcat(string, buffer);
		}
	else if ((keysym == XK_BackSpace) || (keysym == XK_Delete)) {
		if ((length = strlen(string)) > MIN_CHAR) {
			string[length - 1] = NULL;
			XClearWindow(display, *win); 
		}
		else
			XBell(display, 100);
	}

	XDrawString(display, *win, *gc, x, y, string, strlen(string));
	return(0);
}

void gui_clear(win)
Window *win;
/* //////////////////////////////////////////////////////////////
// removes all text from a window
////////////////////////////////////////////////////////////// */
{
	XClearWindow(display, *win);
}

int gui_pending()
/* //////////////////////////////////////////////////////////////////
// Returns number of pending events
////////////////////////////////////////////////////////////////// */
{
	return(XPending(display));
}

void gui_next(report)
XEvent *report;
/* //////////////////////////////////////////////////////////////////
// waits for the next event and stores it in "report"
////////////////////////////////////////////////////////////////// */
{
  int i;

  XNextEvent(display,report);
  if(report->type == ConfigureNotify) {
    for(i = 0;i<Xrtwin;i++){
      if(report->xkey.window == Xrtpos[i].win) {
	if(Xrtpos[i].hide == 1) return;
	Xrtpos[i].x      = report->xconfigure.x;
	Xrtpos[i].y      = report->xconfigure.y;
	Xrtpos[i].width  = report->xconfigure.width;
	Xrtpos[i].height = report->xconfigure.height;
	Xrtpos[i].border = report->xconfigure.border_width;
	return;
      }
    }
  }
}

void gui_wait(sec)
int sec;
/* ///////////////////////////////////////////////////////////////
// waits for the next event without removing it from the queue. The routine 
// waits for a maximum of 'sec' seconds.
// This routine is more or less copied from -XWaitForReadable from the Xlib.
// On the DEC-station, not all include-files are there, so  some things
// are hardcoded.
/////////////////////////////////////////////////////////////// */
{
  struct timeval tv;
/*** MSKCNT is not there, when calculated it is 2 on HP. It cannot hurt 
 *** to assign somewhat more space and be on the safe side ***/ 
/*  unsigned long r_mask[MSKCNT]; */
  unsigned long r_mask[10];
  int result,fd;
  
  tv.tv_sec = sec;
  tv.tv_usec = 0;
/*** Originally   CLEARBITS(r_mask); set the value of all r_mask bits to zero
 *** (from Xlibos.h)                                                ***/
  memset((char *)r_mask,0,10*sizeof(long));
  fd = ConnectionNumber(display);
/*** Originally  BITSET(r_mask,display->fd); set the value of the appropriate 
 *** word in the buffer (see the select man-pages for more detail)   ***/
  r_mask[(fd >> 5)] |= (1<<(fd & 31));
/*** Wait for data until a timeout       ***/
  select(fd+1,r_mask,NULL,NULL,&tv);
}

gui_border(win,bwid)
Window *win;
int bwid;
{
  XSetWindowBorderWidth(display,*win,bwid);
}

