#************************************************
# CVS Log
# client.make,v
# Revision 1.3  1993/10/01  23:58:12  chow
# remove protection problems
#
# Revision 1.2  1993/08/20  18:44:47  chow
# # Added a line to cleanall action for removing mur_generrtab
#
# Revision 1.1.1.1  93/07/14  19:43:21  dbrenner
# UNIDAQ 2.1 distribution - July 14,1993
# 
#************************************************
#===============================================================================
# MAKEFILE DEFINITION:		
#	This makefile builds the MURMUR client (user) library, the MURMUR 
#	server executable, and the MURMUR diagnostic program (MURDER).
#		
# MAKEFILE PRE-REQUESITES:
#	- The following PRE-MAKE environmental variables must be set:
#		PM_LFLAGS ("-lsun" for SGI)
#		
#	- The following products must be setup:
#		fgdbm
#		owcxx -or- objectworks (SunOS only)
#		AnsiCxxFixes           (SunOS only)
#		vxworks                (VxWorks only)
#		
# MAKEFILE TARGETS:
#	fetchfiles - Fetches the includes into $(linc) and source into $(src)/
#
#	all 	   - Builds all executables and libraries into $(src)/ 
#
#	allvxworks - Builds all VxWorks executables and libraries into $(src)/ 
# 
#	copy       - Copies the version deliverable executables, etc, from the 
#	             $(src) into the appropriate areas for a version build.
#
#       vxcopy	   - 'copy' for VxWorks
#
#	destroy    - Removes unwanted files after a version build.  Must be 
#	             called AFTER all component "copy" and "all" targets have
#	             been invoked.
#
#       vxdestroy  - 'destroy' for VxWorks
#
#	clean      - Cleans up after a development build (e.g. removes 
#                    object files).
#
#       vxclean    - 'clean' for VxWorks
#	
#	cleanall   - 'clean + destroy'
#
#===============================================================================

#===============================================================================
# DIRECTORIES
#===============================================================================
linc		= ../include
src		= ../src
lib		= ../lib
bin		= ../bin
inc		= $(MURMUR_CLIENT_DIR)/include
doc		= $(MURMUR_CLIENT_DIR)/doc
etc		= $(MURMUR_CLIENT_DIR)/etc
obj		= $(MURMUR_CLIENT_DIR)/tmp
tmp		= $(MURMUR_CLIENT_DIR)/tmp
tests		= ../tests
tonews		= $(ups)/tonews
toman		= $(ups)/toman
ups		= $(MURMUR_CLIENT_DIR)/ups
SHELL		= /bin/sh
murmur_sccs_dir  	= /usr/products/msrc/murmur
#===============================================================================
# COMMANDS
#===============================================================================
GET		= /usr/bin/get
LINT		= /usr/bin/lint -I$(src)/ -I$(inc) -I/usr/include/bsd

#===============================================================================
# FLAGS
#===============================================================================
DBGFLAG		= -g
INCDIRS		= -I$(linc) -I$(inc) -I$(MURMUR_CLIENT_DIR)/include
ARFLAGS		= r
#
# C flags
#
CC		= cc
CFLAGS		= $(DBGFLAG) $(INCDIRS)
OCFLAGS		= -c $(POCFLAGS) $(CFLAGS)
LDIRS		= -L$(FGDBM_DIR)/lib -L$(lib)
LFLAGS		= $(DBGFLAG) $(LDIRS) -lmuruser $(PM_LFLAGS)
#
# C++ flags
#
CCXX            = CC
CXXFLAGS	= $(DBGFLAG) $(INCDIRS) -I$(ANSICXXFIXES_DIR)/include
LXXDIRS		= $(LDIRS) -L/usr/openwin/lib -L$(MURMUR_DIR)/lib/$(CXX_COMPILER)
LXXFLAGS	= $(DBGFLAG) $(LXXDIRS) -lmuruser -lmessact -ldbint -lfgdbm \
                  -ldisplays -lXm -lXt -lX11 -lm $(PM_LFLAGS)
#
# VxWorks C flags
#
CC68K		= cc68k
C68INCDIRS	= -I$(VXWORKS_DIR)/h $(INCDIRS)
C68FLAGS	= -c $(DBGFLAG) -Dvxworks $(C68INCDIRS)

#===============================================================================
# SUFFIXES
#===============================================================================
.SUFFIXES:
.SUFFIXES: .out .o .c .c~ .y .y~ .l .l~ .s .s~ .csh .csh~ .sh .sh~ .h .h~ \
	.ME .ME~ 

.c.a:;
.f.a:;
.c.ln:;

#===============================================================================
# SOURCE FILES
#===============================================================================
cfiles      =	$(src)/mur_c_user.c	\
		$(src)/mur_c_util.c	\
		$(src)/mur_util.c	\
		$(src)/mur_f_user.c	\
		$(src)/mur_generrtab.c	\
		$(src)/message.c

forfiles    =   $(src)/testclient.F

cxxfiles    =

hfiles      = 	$(inc)/mur_cond.h	\
		$(inc)/mur_def.h	\
		$(inc)/mur_func.h	\
		$(inc)/mur_macros.h	\
		$(inc)/message.h	\
		$(inc)/murmsg_msg_c.h	\
		$(inc)/mur_user.h	\
		$(inc)/mur_f_user.inc	\
		$(inc)/murDspDb.h

diagnostics =	$(src)/murder.c		\
		$(src)/mur_send_message.c

demofiles   =   $(tests)/demo.msg	\
		$(tests)/ssdef.msg	\
		$(tests)/ssdef.inc

docfiles    =

binfiles    =   $(bin)/verify_client	\
		$(bin)/verify_directory \
		$(bin)/verify_component

upsfiles    =   $(ups)/setup.client	\
		$(ups)/unsetup.client

extrafiles  =	$(src)/errtab_input_to.lex	\
		$(src)/errtab_input_to.yacc	\
		$(src)/msg_input_to.lex		\
		$(src)/msg_input_to.yacc	\
		$(inc)/errno.msg		\
		$(inc)/murmur.msg

files       = 	$(cfiles)		\
		$(forfiles)		\
		$(cxxfiles)		\
		$(hfiles)		\
		$(diagnostics)		\
		$(docfiles)		\
		$(demofiles)		\
		$(upsfiles)		\
		$(binfiles)		\
		$(extrafiles)

#===============================================================================
# BUILD THE ENTIRE PRODUCT
#===============================================================================
all :	 	client
	@echo "=== MURMUR CLIENT BUILD : makefile complete ==="	

allvxworks :	vxclient
	@echo "=== MURMUR VXWORKS CLIENT BUILD : makefile complete ==="	

#===============================================================================
# GET SOURCE FILES FROM SCCS
#===============================================================================

$(files) : $(temp_dir)
	- if test -d $(murmur_sccs_dir); then \
				get -p $(murmur_sccs_dir)/s.$(@F) > $@; fi
#===============================================================================
# MAKE CLIENT
#===============================================================================
client:		messageNODB mur_generrtab library diagnostics 
	@echo "=== MURMUR CLIENT BUILD : client build complete ==="	
	@echo

vxclient:	messageNODB  mur_generrtab  vxerrtab vxlibrary 
	@echo "=== MURMUR VXWORKS CLIENT BUILD : client build complete ==="	
	@echo

diagnostics : murder mur_send_message 
	@echo "=== MURMUR CLIENT BUILD : diagnostics build complete ==="	
	@echo

vxerrtab : 
	$(bin)/mur_generrtab $(VXWORKS_DIR)/h/errno.h
	@echo "=== MURMUR CLIENT BUILD : VxWorks errno table generated ==="
	@echo

messageNODB : $(bin)/messageNODB
	@echo " === MURMUR CLIENT messageNODB build complete ==="
	@echo

mur_generrtab : $(bin)/mur_generrtab
	@echo " === MURMUR CLIENT mur_generrtab  build complete ==="
	@echo

#===============================================================================
# MAKE CLIENT LIBRARIES
#===============================================================================
library:	$(lib)/libmuruser.a
	@echo "=== MURMUR CLIENT/SERVER BUILD : client library built ==="	
	@echo

$(lib)/libmuruser.a : 	$(lib)/libmuruser.a(mur_c_user.o)	\
			$(lib)/libmuruser.a(mur_f_user.o)	\
			$(lib)/libmuruser.a(mur_c_util.o)	\
			$(lib)/libmuruser.a(mur_util.o)
	ar $(ARFLAGS) $(lib)/libmuruser.a $?
	/bin/rm $?
	-@ if test -f /usr/bin/ranlib; then ranlib $(lib)/libmuruser.a; fi
	@echo "=== MURMUR CLIENT/SERVER BUILD : user library (libmuruser) built ==="	
	@echo

$(lib)/libmuruser.a(mur_c_user.o) :  	$(src)/mur_c_user.c	\
					$(inc)/mur_cond.h	\
					$(inc)/mur_func.h	\
					$(inc)/mur_def.h	\
					$(inc)/mur_errtab.h	\
					$(inc)/errno_msg_c.h
	$(CC) $(OCFLAGS) $*.c
	@echo

$(lib)/libmuruser.a(mur_f_user.o) :  	$(src)/mur_f_user.c	\
					$(inc)/mur_cond.h	\
					$(inc)/mur_func.h	\
					$(inc)/mur_def.h	\
					$(inc)/murmur_msg_c.h
	$(CC) $(OCFLAGS) $*.c
	@echo

$(lib)/libmuruser.a(mur_c_util.o) :  	$(src)/mur_c_util.c	\
					$(inc)/mur_cond.h	\
					$(inc)/mur_func.h	\
					$(inc)/mur_def.h
	$(CC) $(OCFLAGS) $*.c
	@echo

$(lib)/libmuruser.a(mur_util.o) :  	$(src)/mur_util.c	\
					$(inc)/mur_cond.h	\
					$(inc)/mur_func.h	\
					$(inc)/mur_def.h
	$(CC) $(OCFLAGS) $*.c
	@echo

vxlibrary:	$(lib)/muruser.c68k.o
	@echo "=== MURMUR VXWORKS CLIENT BUILD : client library built ==="	
	@echo

$(lib)/muruser.c68k.o :	$(src)/mur_c_user.c68k.o	\
				$(src)/mur_c_util.c68k.o	\
				$(src)/mur_util.c68k.o		\
				$(src)/murder.c68k.o
	ld68k -o $(lib)/muruser.c68k.o -r $?
	/bin/rm $?
	@echo "=== MURMUR VXWORKS CLIENT BUILD : user library (muruser.c68k.o) built ==="	
	@echo

$(src)/mur_c_user.c68k.o :	$(src)/mur_c_user.c	\
				$(inc)/mur_cond.h	\
				$(inc)/mur_func.h	\
				$(inc)/mur_def.h	\
				$(inc)/mur_errtab.h	\
				$(inc)/errno_msg_c.h
	$(CC68K) $(C68FLAGS) -o $(src)/mur_c_user.c68k.o $(src)/mur_c_user.c
	@echo

$(src)/mur_c_util.c68k.o :      $(src)/mur_c_util.c	\
				$(inc)/mur_cond.h	\
				$(inc)/mur_func.h	\
				$(inc)/mur_def.h
	$(CC68K) $(C68FLAGS) -o $(src)/mur_c_util.c68k.o $(src)/mur_c_util.c
	@echo

$(src)/mur_util.c68k.o :        $(src)/mur_util.c	\
				$(inc)/mur_cond.h	\
				$(inc)/mur_func.h	\
				$(inc)/mur_def.h
	$(CC68K) $(C68FLAGS) -o $(src)/mur_util.c68k.o $(src)/mur_util.c
	@echo

$(src)/murder.c68k.o :          $(src)/murder.c		\
				$(inc)/mur_cond.h	\
				$(inc)/mur_func.h	\
				$(inc)/mur_def.h	\
				$(tests)/demo_msg_c.h
	$(CC68K) $(C68FLAGS) -I$(tests) -o $(src)/murder.c68k.o $(src)/murder.c
	@echo

$(bin)/messageNODB : 	$(src)/message.c 	\
			$(inc)/message.h	\
			$(inc)/murmsg_msg_c.h	\
			$(inc)/murDspDb.h	\
			$(inc)/mur_user.h	\
			$(inc)/mur_macros.h	\
			$(inc)/mur_def.h	\
			$(inc)/mur_cond.h	\
			$(inc)/y.tab.h		\
			$(src)/yNODB.tab.o	\
			$(src)/lexNODB.yy.o
	cc -DMUR_NODB -DMUR_SUNOS $(CFLAGS) $(PCFLAGS) $(OPCFLAGS) \
	$(src)/message.c $(src)/yNODB.tab.o $(src)/lexNODB.yy.o -o $@

$(bin)/mur_generrtab : 	$(src)/mur_generrtab.c	\
			$(inc)/errno_msg_c.h	\
			$(inc)/murmur_msg_c.h	\
			$(inc)/yERR.tab.h	\
			$(src)/yERR.tab.o	\
			$(src)/lexERR.yy.o	
	cc  $(CFLAGS) $(PCFLAGS) $(src)/mur_generrtab.c \
	$(src)/yERR.tab.o $(src)/lexERR.yy.o -o $@

$(src)/yNODB.tab.o : 	$(src)/msg_input_to.yacc
	yacc  -d  $(src)/msg_input_to.yacc
	cc -DMUR_NODB -DMUR_SUNOS $(CFLAGS) $(PCFLAGS) -c $(src)/y.tab.c 
	mv $(src)/y.tab.o $(src)/yNODB.tab.o

$(src)/lexNODB.yy.o : 	$(src)/msg_input_to.lex 
	lex  $(src)/msg_input_to.lex
	cc $(CFLAGS) $(PCFLAGS) -c $(src)/lex.yy.c
	mv $(src)/lex.yy.o $(src)/lexNODB.yy.o

$(inc)/y.tab.h : 	$(src)/msg_input_to.yacc 
	yacc -d $(src)/msg_input_to.yacc 
	rm -rf $(inc)/y.tab.h
	mv y.tab.h $(inc)/y.tab.h

$(src)/yERR.tab.o : 	$(src)/errtab_input_to.yacc
	yacc  -d  $(src)/errtab_input_to.yacc
	cc $(CFLAGS) $(PCFLAGS) -c $(src)/y.tab.c 
	mv $(src)/y.tab.o $(src)/yERR.tab.o

$(src)/lexERR.yy.o : 	$(src)/errtab_input_to.lex 
	lex  $(src)/errtab_input_to.lex
	cc $(CFLAGS) $(PCFLAGS) -c $(src)/lex.yy.c
	mv $(src)/lex.yy.o $(src)/lexERR.yy.o

$(inc)/yERR.tab.h  : 	$(src)/errtab_input_to.yacc
	yacc -d $(src)/errtab_input_to.yacc 
	mv y.tab.h $(inc)/yERR.tab.h

$(inc)/errno_msg_c.h:	$(inc)/errno.msg
	$(bin)/messageNODB -sdb $(inc)/errno.msg

$(inc)/murmur_msg_c.h: $(inc)/murmur.msg
	$(bin)/messageNODB -sdb $(inc)/murmur.msg

$(inc)/mur_errtab.h:	$(inc)/errno_msg_c.h
	$(bin)/mur_generrtab

#===============================================================================
# MAKE MURMUR DIAGNOSTIC ERROR PROGRAM IN SOURCE AREA
#===============================================================================
murder:		$(src)/murder
	@echo "=== MURMUR CLIENT BUILD : diagnostic executable (murder) built ==="	
	@echo

mur_send_message:		$(src)/mur_send_message
	@echo "=== MURMUR CLIENT BUILD : diagnostic executable (mur_send_message) built ==="	
	@echo

testclient:		$(src)/testclient
	@echo "=== MURMUR CLIENT BUILD : diagnostic executable (testclient) built ==="	
	@echo

$(src)/murder:		$(src)/murder.c		\
			$(tests)/demo_msg_c.h	\
			$(lib)/libmuruser.a
	$(CC) $(CFLAGS) -I$(MURMUR_CLIENT_DIR)/tests $(src)/murder.c $(LFLAGS) -o $@

$(src)/testclient:	$(src)/testclient.F	\
			$(inc)/mur_f_user.inc	\
			$(tests)/demo_msg_F.inc	\
			$(tests)/ssdef.inc	\
			$(lib)/libmuruser.a
	f77 $(CFLAGS) -I$(tests) $(src)/testclient.F $(lib)/libmuruser.a \
	    $(LFLAGS) -o $@

$(src)/mur_send_message:		$(src)/mur_send_message.c	\
			$(inc)/murmur_msg_c.h	\
			$(lib)/libmuruser.a
	$(CC) $(CFLAGS) -I$(inc) $(src)/mur_send_message.c $(LFLAGS) -o $@

$(tests)/demo_msg_c.h:	$(tests)/demo.msg
	$(bin)/messageNODB -sdb $(tests)/demo.msg

#===============================================================================
# GET ALL THE FILES FROM SCCS
#===============================================================================
fetchfiles:	$(files)

#===============================================================================
# COPY - USED TO MOVE DELIVERABLES TO THEIR TARGET AREAS
#===============================================================================
copy:
	cp $(src)/murder	$(tests)
	cp $(src)/mur_send_message $(tests)
	mv $(ups)/setup.client $(ups)/setup
	mv $(ups)/unsetup.client $(ups)/unsetup
		
vxcopy:
	mv $(ups)/setup.client $(ups)/setup
	mv $(ups)/unsetup.client $(ups)/unsetup


setprotections:
	cd	$(src);	chmod 644 *.c
	cd 	$(src); chmod 755  premake.client client.make
	cd	$(lib); chmod 644 *.a
	cd 	$(tests); chmod 755 *; chmod 644 *.*
	cd	$(linc); chmod 644 *.h
	cd	$(bin); chmod 755 *
	cd	$(ups); chmod 755 *
	@echo	
	@echo "Correct file protections set"
	@echo

#===============================================================================
# DESTROY - REMOVES LIBRARIES AND EXECUTABLES
#===============================================================================
destroy:
			rm -f $(bin)/murder
			rm -f $(lib)/libmuruser.a
			rm -f $(src)/*.o

vxdestroy:
			rm $(src)/*.c68k.o
			rm $(lib)/muruser.c68k.o

#===============================================================================
# CLEAN
#===============================================================================
clean:
			rm -f $(src)/mur_c_user.o
			rm -f $(src)/mur_c_util.o
			rm -f $(src)/mur_util.o
			rm -f $(src)/murder.o
			rm -f $(src)/mur_send_message.o
			rm -f $(src)/mursrvr.o
			rm -f $(src)/stop_mursrvr.o
			rm -f $(src)/lex*.yy.o
			rm -f $(src)/y*.tab.o
			rm -f $(src)/lex*.yy.c
			rm -f $(src)/y*.tab.c
			rm -f $(src)/mur_generrtab.o
			rm -f $(src)/message.o
			rm -f $(src)/mursrvr
			rm -f $(src)/murder
			rm -f $(src)/mur_send_message
			rm -f $(src)/start_mursrvr
			rm -f $(src)/stop_mursrvr
			rm -f $(tests)/demo_msg_cxx.h
			rm -f $(inc)/*cxx.h
			rm -f $(src)/y.tab.h
			rm -f $(src)/*~
			rm -f $(src)/*#


vxclean:
			rm -f $(src)/mur_c_user.c68k.o
			rm -f $(src)/mur_c_util.c68k.o
			rm -f $(src)/mur_util.c68k.o
			rm -f $(src)/murder.c68k.o
			rm -f $(src)/lex*.yy.o
			rm -f $(src)/y*.tab.o
			rm -f $(src)/lex*.yy.c
			rm -f $(src)/y*.tab.c
			rm -f $(src)/*~
			rm -f $(src)/*#



cleanall: 
			rm -f $(src)/mur_c_user.o
			rm -f $(src)/mur_c_util.o
			rm -f $(src)/mur_util.o
			rm -f $(src)/murder.o
			rm -f $(src)/mur_send_message.o
			rm -f $(src)/mursrvr.o
			rm -f $(src)/stop_mursrvr.o
			rm -f $(src)/lex*.yy.o
			rm -f $(src)/y*.tab.o
			rm -f $(src)/lex*.yy.c
			rm -f $(src)/y*.tab.c
			rm -f $(src)/mur_generrtab.o
			rm -f $(src)/message.o
			rm -f $(src)/mursrvr
			rm -f $(src)/murder
			rm -f $(src)/mur_send_message
			rm -f $(src)/start_mursrvr
			rm -f $(src)/stop_mursrvr
			rm -f $(tests)/demo_msg_cxx.h
			rm -f $(inc)/*cxx.h
			rm -f $(src)/y.tab.h
			rm -f $(src)/*~
			rm -f $(src)/*#
			rm -f $(bin)/murder
			rm -f $(bin)/mur_generrtab
			rm -f $(lib)/libmuruser.a
			rm -f $(src)/*.o
			rm -f $(tests)/murder
			rm -f $(tests)/mur_send_message
