/*************************************************
 * CVS Log
 * mur_util.c,v
 * Revision 1.1.1.1  1993/07/14  19:43:23  dbrenner
 * UNIDAQ 2.1 distribution - July 14,1993
 *
*************************************************/
/* ==========================================================================
**
** MUR_UTIL.C
**
** DESCRIPTION
**	This file contains "C" routines used within both MURMUR client and
**	server libraries.
**
** DEVELOPERS
**	Gary P. Sergey
**
**	Fermilab Data Acquisition Group
**	Batavia, Il 60510, U.S.A.
**
** MODIFICATIONS
**         Date       Initials	Description
**	18-Feb-1992	GPS	Original Entry
** 
** ENTRY POINTS			 SCOPE
**	mur_rec_hdr_pack	external
**	mur_rec_pack		external
**	mur_time_date		external
**      mur_print_msg		external
*/
#include "mur_cond.h"

#ifndef MUR_VXWORKS
#include <time.h>
#include <stdio.h>	/* Need for SGI (__file_s declaration) and stderr */
#endif

#ifdef MUR_VMS
#include "MULTINET_COMMON_ROOT:[multinet.include.rpc]rpc.h"
#include "MULTINET_COMMON_ROOT:[multinet.include.rpc]xdr.h"
#endif

#ifdef MUR_UNIX
#include <rpc/types.h>
#include <rpc/xdr.h>
#endif

#ifdef MUR_VXWORKS
#include <vxWorks.h>
#include <rpc/rpctypes.h>
#include <rpc/xdr.h>
#endif

#include "murmur_msg_c.h"
#include "mur_macros.h"
#include "mur_def.h"
#include "mur_func.h"


/* ==========================================================================
**
** ROUTINE: mur_rec_hdr_pack
*/
#ifdef MUR_STDC
MUR_CLIENT_ERR mur_rec_hdr_pack
(
XDR		    *a_xdrs,	/* IN :  XDR structure */
MUR_PID_PTR	    a_pid,	/* IN : Process ID string */
MUR_NODE_PTR	    a_node,	/* IN : Node name string */
MUR_VERSION_PTR	    a_version,	/* IN : Version number string */ 
MUR_TZ_PTR	    a_zone,    /* IN : Time zone */
long		    *a_reclen	/* IN : User record length */
)
#else
MUR_CLIENT_ERR mur_rec_hdr_pack(a_xdrs, a_pid, a_node, a_version, a_zone, 
                                a_reclen)

XDR		    *a_xdrs;	/* IN : XDR structure */
MUR_PID_PTR	    a_pid;	/* IN : Process ID string */
MUR_NODE_PTR	    a_node;	/* IN : Node name string */
MUR_VERSION_PTR	    a_version;	/* IN : Version number string */ 
MUR_TZ_PTR	    a_zone;     /* IN : Time zone */
long		    *a_reclen;	/* IN : User record length */
#endif
/*
** DESCRIPTION
**	Converts (packs) record header data into standard XDR transport form or
**	converts (unpacks) XDR data back into record header data.  External
**	Data Representation (XDR) routines are utilized to pack/unpack the
**	header.  The User Record header contains the information sent to the
**	MURMUR server upon initial contact from the client.  The header is sent
**	only once (to the server) during the lifetime of a client process.
** 
** VALUES RETURNED
**	MUR_SUCCESS   - User record header packed/unpacked successfully.
**	MUR_SEV_ERR   - Routine failed miserably.
** 
** CALLED BY
**      mur_send
**
** CALLS TO
**	various XDR routines
**
** ========================================================================== 
*/
{ /* begin: mur_rec_hdr_pack */

/*
** USE XDR ROUTINES TO PACK THE USER HEADER DATA ITEMS 
*/

/* Pack MURMUR version # */
if (!(xdr_string(a_xdrs, (char **)&a_version, MUR_VERS_SLEN)))
   {goto mur_err_exit;}

/* Pack process ID */
if (!(xdr_string(a_xdrs, (char **)&a_pid, MUR_PID_SLEN)))
   {goto mur_err_exit;}

/* Pack host node name */
if (!(xdr_string(a_xdrs, (char **)&a_node, MUR_NODE_SLEN)))
   {goto mur_err_exit;}

/* Pack time zone */
if (!(xdr_string(a_xdrs, (char **)&a_zone, MUR_TZ_SLEN)))
   {goto mur_err_exit;}

/* Pack user record length */
if (!(xdr_long(a_xdrs, a_reclen)))
   {goto mur_err_exit;}

/*
** ALL DONE, GOODBYE 
*/

return(MUR_SUCCESS);

mur_err_exit:
return(MUR_SEV_ERR);

} /* end: mur_rec_hdr_pack */


/* ==========================================================================
**
** ROUTINE: mur_rec_pack
*/
#ifdef MUR_STDC
MUR_CLIENT_ERR mur_rec_pack
(
XDR		    *a_xdrs,	    /* IN :  XDR structure */
MUR_MSG_STATUS	    *a_msgcode,	    /* IN :  Message being reported */
MUR_CTIME_PTR	    a_time,	    /* IN :  Client time string */
MUR_CDATE_PTR	    a_date	    /* IN :  Client date string */
)
#else
MUR_CLIENT_ERR mur_rec_pack(a_xdrs, a_msgcode, a_time, a_date)

XDR		    *a_xdrs;	    /* IN :  XDR structure */
MUR_MSG_STATUS	    *a_msgcode;	    /* IN :  Message being reported */
MUR_CTIME_PTR	    a_time;	    /* IN :  Client time string */
MUR_CDATE_PTR	    a_date;	    /* IN :  Client date string */
#endif
/*
** DESCRIPTION
**	Converts (packs) record data into standard XDR transport form or
**	converts (unpacks) XDR data back into record data.  External Data
**	Representation (XDR) routines are utilized to pack/unpack the record.
**	The User Record contains the data sent to the MURMUR server for each
**	message reported.
** 
** VALUES RETURNED
**	MUR_SUCCESS   - User record packed/unpacked successfully.
**	MUR_SEV_ERR   - Routine failed miserably.
** 
** CALLED BY
**      mur_send
**
** CALLS TO
**	various XDR routines
**
** ========================================================================== 
*/
{ /* begin: mur_rec_pack */

/*
** USE XDR ROUTINES TO PACK/UNPACK THE USER RECORD DATA ITEMS 
*/

/* Pack/unpack message code */
if (!(xdr_u_long(a_xdrs, (unsigned long *)a_msgcode)))
   {goto mur_err_exit;}

/* Pack/unpack time */
if (!(xdr_string(a_xdrs, (char **)&a_time, MUR_CTIME_SLEN)))
   {goto mur_err_exit;}

/* Pack/unpack date */
if (!(xdr_string(a_xdrs, (char **)&a_date, MUR_CDATE_SLEN)))
   {goto mur_err_exit;}

/*
** ALL DONE, GOODBYE 
*/

return(MUR_SUCCESS);

mur_err_exit:
return(MUR_SEV_ERR);

} /* end: mur_rec_pack */


/* ==========================================================================
**
** ROUTINE: mur_time_date
*/
#ifdef MUR_STDC
void mur_time_date
(
MUR_CTIME_PTR	a_time,	   /* OUT:  Current time in ASCI */
MUR_CDATE_PTR	a_date		   /* OUT:  Current date in ASCI */
)
#else
void mur_time_date(a_time, a_date)

MUR_CTIME_PTR	a_time;	   /* OUT:  Current time in ASCI */
MUR_CDATE_PTR	a_date;	   /* OUT:  Current date in ASCI */
#endif
/*
** DESCRIPTION
** 	Used to retrieve the current time and date in ASCI strings.
** 
** VALUES RETURNED
**	N/A
** 
** CALLED BY
**      mur_rec_init
**
** CALLS TO
**
** ========================================================================== 
*/
{ /* begin: mur_time_date */

/*---------------------------------------------------------------------------
**
** GET TIME & DATE UNDER VMS AND UNIX
*/
#ifndef MUR_VXWORKS
int	    dummy;
time_t	    ltime;
struct tm   *ltmptr;
char	    *lasc;

/* 
** GET LOCAL TIME/DATE (# SECONDS SINCE 1970 OR WHATEVER ...) 
*/
ltime  = time(0);
if (ltime <= (time_t) 0)
   {
   strcpy((char *)a_time, "--:--:--");
   strcpy((char *)a_date, "--- --- --");
   goto mur_exit;
   }

/*
** CONVERT TO "TM" STRUCTURE FORMAT 
*/
ltmptr = localtime(&ltime);

/*
** CONVERT TO DESIRED ASCII STRINGS 
*/
#ifdef MUR_VMS
lasc = (char *)asctime(ltmptr);
strncpy((char *)a_date, (char *)&lasc[0], 10);
strncpy((char *)a_time, (char *)&lasc[11], 8);
a_time[8]  = (char)0;
a_date[10] = (char)0;
#else
dummy = strftime((char *)a_time, MUR_CTIME_LEN, "%T", ltmptr);
dummy = strftime((char *)a_date, MUR_CDATE_LEN, "%a %b %d", ltmptr);
#endif
#endif

/*---------------------------------------------------------------------------
**
** GET TICK COUNT UNDER VXWORKS
*/
#ifdef MUR_VXWORKS
unsigned long tickcount;

tickcount = tickGet();
sprintf((char *)a_time, "%x", tickcount);
strcpy((char *)a_date, "--- --- --");
#endif

mur_exit:
return;

} /* end: mur_time_date */


/* ==========================================================================
**
** ROUTINE: mur_print_msg
*/
#ifdef MUR_STDC
void mur_print_msg
(
char		*a_bufptr	/* IN :  Message buffer */
)
#else
void mur_print_msg(a_bufptr)

char		*a_bufptr;	/* IN :  Message buffer */
#endif
/*
** DESCRIPTION
**	Used to print the contents of messages sent (or intended to be sent)
**	from the client to the server.
** 
** ========================================================================== 
*/
{
MUR_CLIENT_ERR	err_status;	    /* MURMUR client internal error status */
int		i;
XDR		lxdr;
MUR_PID		lpid;
MUR_NODE	lnode;
MUR_VERSION	lvrsn;
char		*lstrptr;
long int	rec_length;
MUR_ARG_FLAGS	argtypeflags;
MUR_EXTRA_FLAGS extraflags;
char		*argtypeflagsptr, *extraflagsptr;
unsigned int    argtypelen, extralen;
MUR_ARG_TYPE	argtype;
long		arglong;
unsigned long	argulong;
float		argfloat;
MUR_ARG_STRING	argstring;
MUR_APP_TEXT	appstring;
MUR_TEXT_ONLY   textstring;
MUR_PREFIX	prefixstring;
MUR_CTIME	ltime;
MUR_CDATE	ldate;
MUR_TZ		lzone;
MUR_MSG_STATUS	lmsgcode;
MUR_MSG_TYPE_ENUM   lmsgtype;
int toggle;
#define TOGGLE \
 {\
 if(toggle)\
   toggle=0;\
 else\
   toggle++;\
 }
#define TOGGLE_EOL toggle ? "" : "\n"


/* Set XDR buffer to point to message buffer (assume MUR_USER_REC_LEN size) */
xdrmem_create((XDR *)&lxdr, (char *)a_bufptr,
                 MUR_USER_REC_LEN, XDR_DECODE);

/* Unpack the message type */
if (!(xdr_enum((XDR *)&lxdr, (enum_t *)&lmsgtype)))
      {err_status = MUR_CERR_INV_MSGTYPE; goto mur_err_exit;}

switch (lmsgtype)
   {

   /*-------------------------------------------------------------------------
   **
   ** INITIALIZATION MESSAGE
   */

   case MSG_INIT:
      MUR_STDERR "MURMUR INITIALIZATION MESSAGE (MSG_INIT):\n");
      err_status = mur_rec_hdr_pack(&lxdr, lpid, lnode, lvrsn, lzone,
                                    (long *)&rec_length);
      if (err_status != MUR_SUCCESS)
         {
         err_status = MUR_CERR_HDR_PACK;
         goto mur_err_exit;
         }
      MUR_STDERR "   Process ID    : \"%s\"\n", lpid);
      MUR_STDERR "   Node name     : \"%s\"\n", lnode);
      MUR_STDERR "   Version #     : \"%s\"\n", lvrsn);
      MUR_STDERR "   Time zone     : \"%s\"\n", lzone);
      MUR_STDERR "   Record length : %d\n", rec_length);
      break;

   /*-------------------------------------------------------------------------
   **
   ** NORMAL MESSAGE
   */

   case MSG_NORMAL:
#ifdef MURDER_LOCAL
      MUR_STDERR "\n MURMUR NORMAL MESSAGE (MSG_NORMAL) \n");
#else
      MUR_STDERR "\n MURMUR MESSAGE ");
#endif
      err_status = mur_rec_pack(&lxdr, &lmsgcode, ltime, ldate);
      if (err_status != MUR_SUCCESS)
         {
         err_status = MUR_CERR_REC_PACK;
         goto mur_err_exit;
         }
      MUR_STDERR "   <%08X>", lmsgcode);
      MUR_STDERR "   %s %s\n", ltime, ldate);


      argtypeflagsptr = (char *)&argtypeflags.c[0];
      extraflagsptr   = (char *)&extraflags.c[0];


      argtypelen = MUR_NUM_ARGS;
      extralen   = sizeof(long);
      if (!(xdr_bytes(&lxdr, (char **)&argtypeflagsptr, &argtypelen, argtypelen)))
         {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
      if (!(xdr_bytes(&lxdr, (char **)&extraflagsptr, &extralen, extralen)))
          {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}



#ifdef MURDER_LOCAL
      MUR_STDERR "   Argument flags : <");
      for(i=0; i<MUR_NUM_ARGS; i++) 
         {MUR_STDERR "%02X", (unsigned char)argtypeflags.c[i]);}
      MUR_STDERR ">\n");
      MUR_STDERR "   Extra flags    : <");
      for(i=0; i<4; i++) 
         {MUR_STDERR "%02X", (unsigned char)extraflags.c[i]);}
      MUR_STDERR ">\n");
#endif

      /*
      ** PRINT ARGUMENTS (IF ANY) 
      */

      i = 0;
      toggle = 0;

      argtype = (MUR_ARG_TYPE)argtypeflags.c[i];
      while((argtype != MUR_NOTYPE) && (i < MUR_NUM_ARGS))
         {
	 TOGGLE;
         switch(argtype)
            {
            case MUR_LONG:
            if (!(xdr_long(&lxdr, &arglong)))
               {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
 
            MUR_STDERR "  Arg# %1d (long): %12d <%08X>%s",
                    i+1, arglong, arglong, TOGGLE_EOL );
            break;

            case MUR_U_LONG:
            if (!(xdr_u_long(&lxdr, &argulong)))
               {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
            MUR_STDERR "  Arg# %1d (ulong): %11u <%08X>%s",
                    i+1, argulong, argulong, TOGGLE_EOL );
            break;

            case MUR_FLOAT:
            if (!(xdr_float(&lxdr, &argfloat)))
               {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
            MUR_STDERR "  Arg# %1d (float): %15.3g       %s", i+1, argfloat,
		TOGGLE_EOL );
            break;

            case MUR_STRING:
            lstrptr = argstring;
            if (!(xdr_string(&lxdr, (char **)&lstrptr, MUR_ARG_STRING_LEN)))
               {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}

            if(strlen(argstring) > 19)
                {
                MUR_STDERR "%s  Arg# %1d (string): \"%s\"\n",
                    TOGGLE_EOL ,i+1, argstring);
	        toggle = 0;
                }
            else
                {
                
                MUR_STDERR "  Arg# %1d (string): \"%19s\"%s",
                    i+1, argstring, TOGGLE_EOL );                
                }
            break;            

            case MUR_TEXT_ONLY_MSG:
            lstrptr = textstring;
            if (!(xdr_string(&lxdr, (char **)&lstrptr, MUR_TEXT_ONLY_LEN)))
               {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
            MUR_STDERR "   Arg# %1d (text only): \"%s\" \n",
                    i+1, textstring);
            toggle = 0;
            break;            

            default:
            MUR_STDERR "   Argument %1d (Illegal type): %d \n",
                    i+1, argtype);
            break;
            } /* switch(argtype) */

         i++;
         argtype = (MUR_ARG_TYPE)argtypeflags.c[i];
         } /* while */

      /*
      ** PRINT APPENDED TEXT 
      */

	
      TOGGLE;

      if (extraflags.c[MUR_APP_INDEX] == (MUR_BOOL)TRUE)
         {
         lstrptr = appstring;
         if (!(xdr_string(&lxdr, (char **) &lstrptr, MUR_APP_TEXT_LEN)))
            {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
         if(strlen(appstring) > 21)
            MUR_STDERR "%s  Appended text: \"%s\"\n", TOGGLE_EOL , appstring);
         else
            MUR_STDERR "  Appended text: \"%s\"%s", appstring, TOGGLE_EOL);
         }

      TOGGLE;

      if (extraflags.c[MUR_PREFIX_INDEX] == (MUR_BOOL)TRUE)
         {
         lstrptr = prefixstring;
         if (!(xdr_string(&lxdr, (char **) &lstrptr, MUR_PREFIX_LEN)))
            {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
         if(strlen(prefixstring) > 27)
            {
            MUR_STDERR "%s  Appname: \"%s\"\n", TOGGLE_EOL , prefixstring);
            }
         else
            {
            MUR_STDERR "  Appname: \"%s\"%s", prefixstring, TOGGLE_EOL );
            }
         }

      break;

   default:
      MUR_STDERR "\n-- MURMUR ILLEGAL MESSAGE TYPE --: %d \n", lmsgtype);
      break;
   } /* switch */

if(toggle) MUR_STDERR "\n");

return;

mur_err_exit:
mur_internal_err(err_status, 0, (MUR_BOOL)FALSE);
return;
}
