/**************************************************************************
// file:    read_list.c
// author:  C. Timmermans
// version: 1.0
// date:    5/20/93
***************************************************************************/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include "command_structure.h"
#include "commands.h"
#include "files.h"
#include<stdio.h>
#include<string.h>
#include <strings.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/wait.h>

#define _PAR 1
#define _SEQ 2

FILE *c_list;
char line[200];

Command *add_command_next();
Command *add_command_parallel();

int fill_macros()
{
  char fname[100];
  char word[150],macro_opt[150];
  
  macrocount = 0;
/*** Open the file ***/
  if(decode_filename(COMM_LIST,fname) < 0) {
    return(-1);
  }
  if((c_list = fopen(fname,"r")) == NULL){
    return(-1);
  }
/*** Start reading in Macro's ***/
  while(((int) fgets(line,199,c_list))  != 0) {
    strcpy(macro_opt,"");
    if(sscanf(line,"%s %s",word,macro_opt) > 0) {
      if(word[0] == '#') {
#ifdef _DEBUG
    printf("%s",line);
#endif
	read_macro(&word[1],macro_opt);
      }
    }
  }
  fclose(c_list);
  return(0); 
}


int read_macro(macro_name,macro_opt) 
char *macro_name,*macro_opt;
{
  char *c,*cw,word[50];
  int order,i_sub;
  Command *pt_com,*pt_tmp,*pt_start;

/*** Initialization ***/
  pt_com = NULL;
  pt_start = NULL;
  word[0] = '\0';
  i_sub = 0;

/*** First find the start point of the Macro definition ***/
  while((c = strchr(line,'{')) == (char *)NULL){
    while(((int)fgets(line,199,c_list)) != 0) {
      if(sscanf(line,"%s",word) > 0)
	if(strcmp(word,"C") != 0) break;
    }
  }
  if(c == NULL) {
    printf("Cannot find a {\n");
    return(-1);
  }
/*** Check out if Macro is to be executed sequentially or in parallel ***/
  *c = '\0';
  if(strstr(line,"SEQUENTIAL")    != (char *)NULL) order = _SEQ;
  else if(strstr(line,"PARALLEL") != (char *)NULL) order = _PAR;
  else return(-1);
/*** Make sure it is possible to read the remainder of the line ***/
  c++;
  strcpy(line,c);
/*** The body of the macro, read until a '}' is found.  ***/
  do {
/*** first make sure that the line is not commented out ***/
    do {
      if(sscanf(line,"%s",word) > 0) {
	if(strcasecmp(word,"C") != 0) break;
      }
      c = fgets(line,199,c_list);
      if(c == NULL) return(-1);
    }while(c != NULL);

    if(strchr(line,'{') != (char *)NULL) {
/*** A sub-part of the Macro, this will be a new Macro ***/
      sprintf(word,"%%%s.%d",macro_name,i_sub);
      if(read_macro(&(word[1]),macro_opt) < 0) return(-1);
      i_sub++;
/*** Add a call to the sub-macro in this macro ***/
      sprintf(line,"%s %s",word,macro_opt);
    }
/*** add a command to the macro ***/
    if(sscanf(line,"%s",word) > 0) {
      if(word[0] == '#') return(-1);
      if(strcasecmp(word,"C")) {
	cw = strstr(line,word);
	if((c = strchr(line,'}')) != (char *)NULL) {
	  *c = '\0';
	}
	if( c != cw) {
	  if(order == _SEQ) pt_tmp = add_command_next(pt_com);
	  else              pt_tmp = add_command_parallel(pt_com);
	  if(pt_tmp != NULL){
	    pt_com = pt_tmp;
	    if(pt_start == 0) pt_start = pt_com;
	  }
	}
	if(c!=NULL) break;
      }
    }
  }while(((int)fgets(line,119,c_list)) != 0);
/*** Check if a '}' was found ***/
  if(c== NULL) return(-1);
/*** add this macro to the macro-list ***/
#ifdef _DEBUG
  printf("Adding MACRO !!%s!! to the list\n",macro_name);
#endif
  if(macrocount < MAX_MACRO) {
    strcpy(macrotbl[macrocount].name,macro_name);
    strcpy(macrotbl[macrocount].opt,macro_opt);
    macrotbl[macrocount].ptr = pt_start;
    macrocount+=1;
  }
  return(0);
}

extract_commands(comm,opt)
char comm[3][100],opt[3][100];
{
  char *c_if,*c_else;
  char *c_op,*c_cl;
  int i;
  char word[50],*c_opt,*c_tmp;


/*** Initialize it all ***/
  for(i=0;i<3;i++) {
    comm[i][0] = '\0';
    opt[i][0] = '\0';
  }
  if((c_tmp = strchr(line,'\n')) != (char *)NULL) *c_tmp = '\0';
  if((c_if = strstr(line,"IF")) != (char *)NULL){
    if((c_op = strchr(line,'(')) == (char *)NULL) return(-1);
    if((c_cl = strchr(line,')')) == (char *)NULL) return(-1);
    c_op+=1;
    *c_cl = '\0';
    c_cl+=1;
    if((c_else = strstr(c_cl,"ELSE")) != (char *)NULL) {
      *c_else = '\0';
      c_else+=4;
    }
  }
/*** No IF statement ***/
  if(c_if == NULL){
    if(sscanf(line,"%s",word) <=0) return(-1);
    strcpy(comm[0],word);
    c_tmp = strstr(line,word);
    c_tmp+=strlen(word);
    if(sscanf(c_tmp,"%s",word) <= 0) return(0);
    c_opt = strstr(c_tmp,word);
    strcpy(opt[0],c_opt);
    return(0);
  }
/*** Now the IF statement ***/
  if(sscanf(c_op,"%s",word) <=0) return(-1);
  strcpy(comm[0],word);
  c_tmp = strstr(c_op,word);
  c_tmp+=strlen(word);
  if(sscanf(c_tmp,"%s",word) > 0) {
    c_opt = strstr(c_tmp,word);
    strcpy(opt[0],c_opt);
  }
/*** The TRUE part ***/
  if(sscanf(c_cl,"%s",word) > 0) {
    strcpy(comm[1],word);
    c_tmp = strstr(c_cl,word);
    c_tmp+=strlen(word);
    if(sscanf(c_tmp,"%s",word) > 0) {
      c_opt = strstr(c_tmp,word);
      strcpy(opt[1],c_opt);
    }
  }
/*** and the FALSE part ***/
  if(c_else == NULL) return(0);
  if(sscanf(c_else,"%s",word) <=0) return(0);
  strcpy(comm[2],word);
  c_tmp = strstr(c_else,word);
  c_tmp+=strlen(word);
  if(sscanf(c_tmp,"%s",word) > 0) {
    c_opt = strstr(c_tmp,word);
    strcpy(opt[2],c_opt);
  }
  return(0);
}


Command *add_command_next(pt)
Command *pt;
{
  char com[3][100], opt[3][100];
  Command *nx, *sp;
	
  if(extract_commands(com,opt) < 0) {
    return(NULL);
  }
/*** find and fill the next empty spot in the command_space ***/
  sp = cmdtbl;
  while(*(sp->command) != '\0') sp+=1;
  strcpy(sp->command,com[0]);
  strcpy(sp->option,opt[0]);
/*** add this to the Macro ***/
  if(pt != NULL) {
    if(pt->next == NULL) pt->next = sp;
    else{
      nx = pt->next;
      while(nx->parallel != NULL) nx = nx->parallel;
      nx->parallel = sp;
    }
  }
  nx = sp;
  if(com[1][0] != '\0') {
    while(*(sp->command) != '\0') sp+=1;
    strcpy(sp->command,com[1]);
    strcpy(sp->option,opt[1]);
    nx->true = sp;
  }
  if(com[2][0] != '\0') {
    while(*(sp->command) != '\0') sp+=1;
    strcpy(sp->command,com[2]);
    strcpy(sp->option,opt[2]);
    nx->false = sp;
  }
  return(nx);
}



Command *add_command_parallel(pt)
Command *pt;
{
  char com[3][100], opt[3][100];
  Command *nx, *sp;
	
  if(extract_commands(com,opt) < 0) return(NULL);
/*** find and fill the next empty spot in the command_space ***/
  sp = cmdtbl;
  while(*(sp->command) != '\0') sp+=1;
  strcpy(sp->command,com[0]);
  strcpy(sp->option,opt[0]);
/*** add this to the Macro ***/
  if(pt != NULL) {
    if(pt->parallel == NULL) pt->parallel = sp;
    else{
      nx = pt->parallel;
      while(nx->parallel != NULL) nx = nx->parallel;
      nx->parallel = sp;
    }
  }
  nx = sp;
  if(com[1][0] != '\0') {
    while(*(sp->command) != '\0') sp+=1;
    strcpy(sp->command,com[1]);
    strcpy(sp->option,opt[1]);
    nx->true = sp;
  }
  if(com[2][0] != '\0') {
    while(*(sp->command) != '\0') sp+=1;
    strcpy(sp->command,com[2]);
    strcpy(sp->option,opt[2]);
    nx->false = sp;
  }
  return(nx);
}

int execute_shell(name,opts)
char *name, *opts;
{
  char sh_command[200];

#ifdef _DEBUG
  printf("Executing shell !!%s!! !!%s!!\n",name,opts);
#endif
  sprintf(sh_command,"%s %s",name,opts);
  system(sh_command);
  return(1);
}

int execute_command(name,opts)
char *name,*opts;
{
  int i;

#ifdef _DEBUG
  printf("Executing command !!%s!! !!%s!!\n",name,opts);
#endif
  for(i = 0;i<cmdcount;i++){
    if(strcmp(name,cmdtable[i].com) == 0) 
      return((*cmdtable[i].act)(opts));
  }
  return(-111);
}

int execute_macro(name,opts)
char *name,*opts;
{
  int i;
  
#ifdef _DEBUG
  printf("Executing macro !!%s!! !!%s!!\n",name,opts);
#endif
  for(i=0;i<macrocount;i++){
    if(strcasecmp(name,macrotbl[i].name) == 0) {
      return(exec_cmd(macrotbl[i].ptr,macrotbl[i].opt,opts));
    }
  }
  return(-111);
}

int exec_cmd(ptr,opt,sub)
Command *ptr;
char *opt;
char *sub;
{
  int pid = 0;
  int c_code,r_code = 1;
  Command *nxt;

/*** There are only parallel OR sequential parts, it is never mixed (macro's
     take care of that) ***/ 
  nxt = ptr;
  if(ptr->parallel != NULL){
    while((pid >=0) && (nxt != NULL)) {
      if((pid = (int)fork()) == 0) {
	if(exec_this_cmd(nxt,opt,sub) < 0) exit(1);
	exit(0);
      }
      if(pid > 0) nxt = nxt->parallel;
    }
/*** wait for all children to finish ***/
    while(wait_for_child(&c_code) > 0) {
      if(WIFEXITED(c_code)) {
	c_code = WEXITSTATUS(c_code);
	if(c_code != 0) r_code = -1;
      }
    }
/*** if they are not all done, execute the rest sequential ***/
    while(nxt != NULL){
      if(exec_this_cmd(nxt,opt,sub) < 0) r_code = -1;
      nxt = nxt->parallel;
    }
    return(r_code);
  }
  if(ptr->next != NULL) {
    while(nxt != NULL){
      if(exec_this_cmd(nxt,opt,sub) < 0) return(-1);
      nxt = nxt->next;
    }
    return(1);
  }
  return(exec_this_cmd(ptr,opt,sub));
}

int exec_this_cmd(ptr,opt,sub)
Command *ptr;
char *opt;
char *sub;
{
  char *c,command[100],option[100],temp[100];
  int r_code;

/*** replace occurances of opt with sub  ***/
  strcpy(command,ptr->command);
  strcpy(option,ptr->option);
  if(strlen(opt) > 0) {
    if(strcmp(command,opt) == 0) strcpy(command,sub);
    while((c = strstr(option,opt)) != (char *)NULL){
      *c = '\0';
      strcpy(temp,option);
      strcat(temp,sub);
      c+=strlen(opt);
      strcat(temp,c);
      strcpy(option,temp);
    }
  }
/*** actually execute the command ***/
  if(command[0] == '%')      r_code = execute_macro(&command[1],option);
  else if(command[0] == '@') r_code = execute_shell(&command[1],option);
  else                       r_code = execute_command(command,option);
  if(r_code < 0) return(-1);
/*** execute the true/false results as well ***/
  if(r_code == 0 && ptr->false != NULL) 
    return(exec_this_cmd(ptr->false,opt,sub));
  if(r_code == 1 && ptr->true != NULL) 
    return(exec_this_cmd(ptr->true,opt,sub));
  return(r_code);
}

remove_macros()
{
  int i;

  macrocount = 0;
/*** remove all commands out of the command-space ***/
  for(i=0;i<MAX_COMMAND;i++){
    cmdtbl[i].command[0] = '\0';
    cmdtbl[i].option[0] = '\0';
    cmdtbl[i].parallel = NULL;
    cmdtbl[i].next = NULL;
    cmdtbl[i].true = NULL;
    cmdtbl[i].false = NULL;
  }
}
