/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <unistd.h>
#include <string.h>
#include<stdio.h>
#include<sys/types.h>
#include<sys/ipc.h>
#include<sys/shm.h>
#include<sys/time.h>
#include <stdlib.h>
#include "runco.h"
#include "files.h"
#include "sdc_pack_msg_c.h"

#define PIPE_READ 0
#define PIPE_WRITE 1

extern int runstate;
extern int wait_for;
extern char re_prompt[];
extern char collector[];
extern char recorder[];
/*****************************************************/
/* some variables for general use                    */
/*****************************************************/

int pipe_io[2];
struct timeval t_data;
extern char ownpname[];

/***********************************************************/
void Init_Process(argc,argv)
int argc;
char **argv;
{
  int pid = -1;
  short s;

/* //////////////////////////////////////////////////////////////////
// The global variables are stored in shared memory. If a process
// gets killed it can be restarted and all information is still
// available.
//////////////////////////////////////////////////////////////// */
  if(strcmp(ownpname,"runco") != 0) {
    Send_Message(ownpname,"EXIT");
    return;
  }
  if(pipe(pipe_io) < 0) {
    printf("Cannot open a pipe\n");
    exit(2);
  }
  write(pipe_io[PIPE_WRITE],(char *)&pid,sizeof(int));
  wait_for = 0;
  set_value_in_sp("wait_for",(char *)(&wait_for));
  Crit_Var("collector");
  Crit_Var("recorder");
  Crit_Var("operator");
  read_value_from_sp(collector,"runstate",&s,(char *)(&runstate));
  remove_macros();
  t_data.tv_sec = 0;
}
/***********************************************************/
int Process_Event(string)
char *string;
{
  char proc_send[30],rest[100],com1[100],command[100];
  int pid,i;

  if(strcmp(ownpname,"runco") != 0) {
    return(0);
  }
#ifdef _DEBUG 
	printf("Entering process event with command\n %s\n",string);
#endif 
/* ////////////////////////////////////////////////////////////////
// get the important information out of the command string
/////////////////////////////////////////////////////////////// */
  Parse(string,proc_send,command);
  Parse(command,com1,rest);
  if(strcmp(com1,"RunPar") == 0) {
    if(strncmp(proc_send,"operator",8) == 0){
      wait_for = wait_for | QP_BO_RUNPAR;
      set_value_in_sp("wait_for",(char *)(&wait_for)); 
    }
  }
  else if(strcmp(com1,"Storage") == 0) {
    if(strncmp(proc_send,"operator",8) == 0){ 
      wait_for = wait_for | QP_BO_STORAGE;
      set_value_in_sp("wait_for",(char *)(&wait_for)); 
    }
  }
  else if(strcmp(com1,"re:prompt") == 0) {
    strcpy(re_prompt,rest);
    set_value_in_sp("re_prompt",re_prompt);
    wait_for = wait_for | QP_OTHER;
    set_value_in_sp("wait_for",(char *)(&wait_for)); 
  }
  else if(com1[0] == '@') {
    sprintf(command,"%s %s &",&(com1[1]),rest);
    system(command);
  }
  else if(strcmp(com1,"ping") == 0) {
    return(0);
  }
  else {
    pid = 0; 
    if((pid = (int)fork()) == 0) {
      pid = (int)getpid();   
/* //////////////////////////////////////
// Child executes the commands
///////////////////////////////////// */
      if((i=execute_command(com1,rest)) == -111) {
#ifdef _DEBUG
	printf("command not found; fill macro;\n");
#endif
	fill_macros();
#ifdef _DEBUG
	printf("Trying to execute MACRO %s\n",com1);
#endif
	if((i = execute_macro(com1,rest)) == -111) {
/* ///////////////////////////////////////////////////////////
// I don't know what kind of command this is
//////////////////////////////////////////////////////////// */
#ifdef _DEBUG
	  printf("command is no macro; send message back;\n");
#endif
	  add_err_string(com1);
	  send_err_string(Runco_NotDone);
	}
	remove_macros();
      }
      if((i<0) && (i != -111)){
/* ////////////////////////////////////////////////////////
// Let the logbook know that there are errors
/////////////////////////////////////////////////////// */
	add_err_string(com1);
	send_err_string(Runco_No_Xecute);
      }
      write(pipe_io[PIPE_WRITE],(char *)&pid,sizeof(int));
      exit(0); 
    } 
    if(pid == -1) {
#ifdef _DEBUG
      printf("Cannot execute ANY commands\n");
#endif
      send_err_string(Runco_ANY_comm);
      return(0);
    } 
  }
  return(0);
}
/***********************************************************/
void No_Event()
{
  int pid,sta;
  int eventnr, maxnevents,rstate;
  short s;
  struct timeval tv;
  struct timezone tz;
  
  if(strcmp(ownpname,"runco") != 0) {
    idle(1);
    return;
  }
/* ///////////////////////////////////////////////////////////////
// Check if child processes are ready
/////////////////////////////////////////////////////////////// */
  if (read(pipe_io[PIPE_READ],(char *)&pid,sizeof(int)) == sizeof(int)) {
    if (pid > 0) pid = wait_for_child(&sta);
    else         write(pipe_io[PIPE_WRITE],(char *)&pid,sizeof(int));
  }

/* ////////////////////////////////////////////////////////////////
// Be sure to stop the run when needed
//////////////////////////////////////////////////////////////// */
  gettimeofday(&tv,&tz);
  if((tv.tv_sec - t_data.tv_sec) > DATATIME) {
    read_value_from_sp(collector,"runstate",&s,(char *)(&rstate));
    if(rstate != PAUSE_STATE   &&   rstate != END_STATE) {
      read_value_from_sp(collector,"eventnr",&s,(char *)(&eventnr));
      read_value_from_sp(collector,"maxnevents",&s,
			 (char *)(&maxnevents));
      if((eventnr >=maxnevents) && (maxnevents > 0)) 
	Send_Message(ownpname,"SUSPEND_RUN");
    }
    gettimeofday(&t_data,&tz);
  }
  else rstate = runstate;
  idle(1);
  if (runstate != rstate){
    read_value_from_sp(collector,"runstate",&s,(char *)(&rstate));
    if((rstate ==END_STATE) && (runstate != END_STATE)){ 
      Send_Message(ownpname,"STOP_RUN");
    }
    runstate = rstate;
    set_value_in_sp("runstate",(char *)(&runstate));
  }
}
/***********************************************************/
void Exit_Process()
{

}
/***********************************************************/
