#include <stdio.h>
#include <stdlib.h>
#include <string.h>
/* -------------------------------------------------------------
   Program:  ndmake
   Purpose:  To parse a text file containing a list of .C/F files
      along with a list of routines contained in those .C/F files
      to be (1) non-dynamically linked and then (2) 'Add'ed to
      the list of accessible commands running under analyzer.
      The name of the file-to-be-parsed is given as the argument
      to ndmake.  (If not supplied, the named defaults to NDMakefile).

      The format of the file-to-be-parsed is given as follows:
      
      filename_A.c routine_A1 routine_A2 ... routine_An;
      filename_B.c routine_B1 routine_B2 ... routine_Bn;
      .
      .
      .

      The '#' symbol, when used as the first character of a line,
      introduces a comment.
      The '*' symbol, as the first character of a line, indicates
      that the line is to be printed literally to stdio during
      the ndmake.


      When ndmake is run, two things are accomplished.
      (1)  The non-dynamic makefile is invoked.  The filename's parsed
           from the NDMakefile are passed to the make and are linked
	   to the executable analyzer.
      (2)  Creates file NDanalyzer.h -- a header file which is included
           in analyzer_procs.c, which contains the names of functions
           to be non-dynamically linked.
   ------------------------------------------------------------- */


#define MAX_DONTPASS      6           /* max no. args not passd*/
#define HEADERSIZE        10000

main(argc, argv)
int argc;
char *argv[];
/* -------------------------------------------------------------
Procedure:  main
Description:  
   ------------------------------------------------------------- */
{
  FILE *stream;
  char filename[512];
  char makefile_name[512];
  char makearg[2048];
  char makearg_O[2048];                 /* to build $(NDLINKO) which is
					   concatntd onto makearg */
  char file_line[512];
  char line_word[512];
  char full_name[512];                  /* use with decode_filename() */
  char header[HEADERSIZE];
  char headlist[2000];                  /* function names to go in header
					   separated by NULL.  */
  char *PTheadpos=headlist;
  char *PTline=NULL, *PTword=NULL;
  int args_ok;                          /* value rtnd by fscanf */
  int dont_pass[MAX_DONTPASS+2];        /* gives argv indices NOT to 
					   pass as args to mkfile*/
  int pass_count=0;                     /* index to dont_pass */
  char scratch[128];

  /* Zero important automatic variables
   */
  memset(dont_pass, 0, sizeof(dont_pass));

  /* Get Default values for Filenames
   */
  strcpy(makefile_name, "Makefile");
  strcpy(filename, "NDMakefile");


  /* Print Help Message (if first arg is -h) and exit.
   */
  if (argc >= 2) {
   if (!strcmp(argv[1], "-h"))
    {
      printf("\nNon-Dynamic Make (V1.1, Nov 17, 1994)\n");
      printf("          Eric Babe with mods by Bob Ball\n");
      printf("Purpose:  To parse an input ND-makefile, passing the modules\n");
      printf("          to a 'make' call, and to write the parsed content \n");
      printf("          of the ND-makefile to file NDanalyzer.h .\n");
      printf("Flags:\n");
      printf("   -mf Makefile (default: %s)\n", makefile_name);
      printf("   -nd Non-Dynamic Makefile (default: %s)\n", filename);
      printf("   -h  Show this help message\n");
      printf("   All other arguments passed as arguments to make.\n\n");
      printf("NDMakefile syntax:\n");
      printf("   filenameA functionA1 functionA2 ... functionAn\n");
      printf("   filenameB functionB1 functionB2 ... functionBn\n");
      printf("   etc...\n");
      printf("   # -- introduces a comment\n");
      printf("   * -- line is printed to stdio during ndmake\n\n");
      exit(0);
    }
  }

  /* Check arg list to see if there are any switches to change Filenames 
   */
  {
    int i;
    char expect=0;            /* 1...makefile name;   2...NDmakefile name */
    for (i=1; i<argc && pass_count<MAX_DONTPASS; i++)
      {
	switch (expect)
	  {
	    case 1: 
	      strcpy(makefile_name, argv[i]); 
	      dont_pass[pass_count++]=i; 
	      break;
	    case 2: 
	      strcpy(filename, argv[i]); 
	      dont_pass[pass_count++]=i; 
	      break;
	    case 3:
	      break;
	  }    
	expect=0;
	if (!strcmp(argv[i], "-mf")) 
	  {expect=1; dont_pass[pass_count++]=i;}
	else if (!strcmp(argv[i], "-nd")) 
	  {expect=2; dont_pass[pass_count++]=i;}
      }
  }


  /* Open and parse ndmakefile
   */
  switch (decode_filename(filename, full_name))
    {
      /* see utillib.c in $TOPUNIDAQ/lib for desc of return codes */
       case -2:      /* filename should be OK in itself */
          strcpy(full_name, filename);
	  stream=fopen(full_name, "r");
	  break;
	case 0:      /* decode OK */
	  stream=fopen(full_name, "r");
	  break;
	default:     /* error */
	  stream=NULL;
	  break;
    }
  if (stream)
    {
      int i,j;

      strcpy(makearg, "make -f ");
      strcat(makearg, makefile_name);
      strcat(makearg, " ");
      strcat(makearg, " ");
      
      /* Add argv's onto makearg (excluding dont_pass indices)
       */
      for (i=1; i<argc; i++)
	{
	  for (j=0; j<MAX_DONTPASS; j++)
	    if (dont_pass[j]==i) goto skip_pass;
	  strcat(makearg, argv[i]);
          strcat(makearg, " ");
	  skip_pass:
	  continue;
	}

      strcat(makearg, "NDLINK=\"");
      strcpy(makearg_O, "NDLINKO=\"");

      printf("\nParsing Non-Dynamic Makefile: %s\n", filename);
      fscanf(stream, "%*[\n]");      /* Eliminates whitespace */
      /* Scan lines of NDMakefile until there are none left 
       */
      while ((args_ok=fscanf(stream, "%[^\n]%*c", file_line))!=EOF)
	if (args_ok==1)
	{
	  PTline=file_line;
/*  
 *  Trim trailing blanks
 */
	  for (j=strlen(PTline)-1; j>=0; j--)  {
	    if (*(PTline+j)==' ' || *(PTline+j)=='\n')  *(PTline+j)='\0';
	    else                                        break;
	  }
/*	  printf("* -%s-\n", PTline);   */

	  /* Get first non-whitespace in PTline */
	  while (*PTline==' ' || *PTline=='\t') PTline++;
	  if (*PTline=='*')                    /* print line */
	    {
	      printf("%s\n",PTline);
	    }
	  else if (*PTline!='#')               /* not comment line */
	    {
	      char word_done=1;
	      char added_function=0;
	      char fortran_flag=0;
	      char extension_flag=0;

/*	      printf("%s\n",PTline);     */
	      strcpy(line_word,"");
	      /* Parse name from file_line to line_word breaking on space or CR */
	      for (PTword=line_word; *PTline!=' ' && *PTline!='\n' && *PTline; 
		   PTline++, PTword++)
		{
		  *PTword = *PTline;
		  if (*PTword=='/') extension_flag=0;
		  else if (*PTword=='.') extension_flag=1;
		  fortran_flag = (extension_flag && *(PTword-1)=='.' && 
				  (*PTword=='f' || *PTword=='F'));
		}
	      *PTword=0;

	      strcat(makearg, line_word);
	      /* If line_word does not end in an extension, append '.f' */
	      if (!extension_flag) 
		{
		  strcat(makearg, ".f");
		  fortran_flag=1;
		  /* add lineword to makearg_O plus a .o extension */
		  strcat(makearg_O, line_word);
		  strcat(makearg_O, ".o");
		}
	      else {
		/* add lineword to makearg_O; change extension to .o */
		strcat(makearg_O, line_word);
		makearg_O[strlen(makearg_O)-2]='.';
		makearg_O[strlen(makearg_O)-1]='o';
	      }
	      strcat(makearg_O, " ");
	      /* Test fortran extension */


	      strcat(makearg, " ");
	      printf(" --> %s\n", line_word);

	      /* Add args to header list */
	      while(*PTline!='\n' && *PTline)
		{
		  if (*PTline!=' ') 
		    {
		      *PTheadpos = *PTline;
		      PTheadpos++; 
		      word_done=0; added_function=1;
		    }
		  else if (!word_done) 
		    {
		      if (fortran_flag)
			{
			  *PTheadpos='_';
			  PTheadpos++;
			}
		      *PTheadpos=0; 
		      PTheadpos++; 
		      word_done=1;
		    }
      		  PTline++;
		}
	      if (added_function && fortran_flag)
		{
		  *PTheadpos='_';
		  PTheadpos++;
		}
	      if (!added_function)
		{
		  char *src;
		  for (src=line_word; *src!=' ' && *src!='\n' && *src!='.'; 
		       src++, PTheadpos++)
		    *PTheadpos = *src;
      		}
	      *PTheadpos=0;
	      PTheadpos++;
	    }
	  
            fscanf(stream, "%*[\n]");
	}
      strcat(makearg, "\" ");
      strcat(makearg_O, "\"");
      strcat(makearg, makearg_O);

      /* Build onto header from headlist */
      {
	char scratch[128];
	char *PTheadword=headlist;

	/* Setup header initial text 
	 */
	strcpy(header, "/*******************************************\n");
	strcat(header, " *                                         *\n");
	strcat(header, " * Header file NDanalyzer.h for non-       *\n");
	strcat(header, " *    dynamic analyzer.  Declares external *\n");
	strcat(header, " *    all routines to be called, and adds  *\n");
	strcat(header, " *    them to the appropriate structure.   *\n");
	strcat(header, " *                                         *\n");
	strcat(header, " *******************************************/\n");
	strcat(header, "\n\n");
	strcat(header, "/* --------------------------------------------------------- */\n");
	strcat(header, "/* Routines which ana_nd will call for each event            */\n");
	strcat(header, "/* This is compiled into analyzer_procs.c                    */\n");
	strcat(header, "/* --------------------------------------------------------- */\n");

	/* Build function prototypes */
	while (PTheadword<PTheadpos)
	  {
	    if (*PTheadword)
	      {
		sprintf(scratch, "extern void %s();\n", PTheadword);
		strcat(header, scratch);
	      }
	    while (*PTheadword && PTheadword<PTheadpos) PTheadword++;
	    PTheadword++;
	  }
	/* Build command array */
	strcat(header, "struct void_function_struct {\n");
	strcat(header, "     void    (*PTfunc)();\n}\n");
	strcat(header, "nd_func_list[64]={\n");
	PTheadword=headlist;
	while (PTheadword<PTheadpos)
	  {
	    if (*PTheadword)
	      {
		sprintf(scratch, "        %s,\n", PTheadword);
		strcat(header, scratch);
	      }
	    while (*PTheadword && PTheadword<PTheadpos) PTheadword++;
	    PTheadword++;
	  }	
	strcat(header, "        NULL,\n");
	strcat(header, "};");

	/* Write Header to file 
	 */
	{
	  FILE *stream2;
	  stream2=fopen("NDanalyzer.h", "w");
	  if (stream2) 
	    fprintf(stream2, "%s\n", header);
	  else {
	    printf("Error writing to NDanalyzer.h.\n");
	    exit(0);
	  }
	  fclose(stream2);
	}
      }

      printf ("\n");
      /* Write NDMakefile name to NDLinkName 
       */

/*  Commented out, 11/17/94, B. Ball, no longer used.
      {
	FILE *stream2;
	switch (decode_filename("NDLinkName", full_name))
	  {
	    * see utillib.c in $TOPUNIDAQ/lib for desc of return codes *
	  case 0:      * decode OK *
	    stream=fopen(full_name, "r");
	    break;
	  default:     * error *
	    stream=NULL;
	    break;
	  }
	stream2=fopen("NDLinkName", "w");
	if (stream2) 
	    fprintf(stream2, "%s\n", filename);
	else {
	  printf("Error writing to NDLinkName.\n");
	  exit(0);
	}
	fclose(stream2);
      }
 */

      /* Perform Make */
      printf("%s\n", makearg);
      if (system(makearg))
	{
	 /*
	   Commented out because system() returns an error message
	   even though everything appears to be working fine.
	    printf("Terminal Error:  command processor not available.\n");
	    exit(0);
	  */
	}
    }
  else
    {
      printf("Error opening Non-dynamic makefile: %s\n", full_name);
      exit(0);
    }

}
