/*************************************************
 * uni_config.c,v
*************************************************/
/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/
/**********************************************
 * Charles Timmermans, University of Michigan *
 * Graphical interface to configure UNIDAQ    *
 **********************************************/
#include <stdlib.h>
#include <stdio.h>
#include "uni_config.h" 
#include <X11/Xlib.h>

#define MAX_START 20
/*** general variables ***/

char editor[100];
char gen_txt[100];
struct cnf_type bt[MAX_START];
int n_but;
int ex_flag;

/*** Windows and GC ***/
Window uc,   uc_but[CNF_MAX];
GC     ucGC, uc_butGC[CNF_MAX];
Window st,   st_but[MAX_START];
GC     stGC, st_butGC[MAX_START];
XEvent report;

main()
{
  char tmp[100],buf[100];
/*************************************************
 Check the basics:
   1. Is USERUNIDAQ known
   2. Which text editor should be used
 *************************************************/
  if(getenv("USERUNIDAQ") == NULL) {
    printf("Please source the UNIDAQ setup.csh before running uni_config\n");
    exit(0);
  }
  if(getenv("EDITOR") == NULL) strcpy(editor,"vi");
  else strcpy(editor,getenv("EDITOR"));
  printf("Enter your text editor [%s] ",editor);
  gets(tmp);
  printf("\n");
  if(sscanf(tmp,"%s",buf) == 1) strcpy(editor,buf);
/**************************************************
 Create the uni_config display. Use the gui_lib
 developed when creating operator.
 **************************************************/
  create_uni_config();
  while(1){
    gui_next(&report);
    update_uni_config();
  }
}

create_uni_config()
{
  int i;
  int uc_y,but_x,but_y;

  if(CNF_MAX < cnf_nr) {
    printf("Error in uni_config configuration\n");
    printf("Please get an expert to repair this\n");
    exit(1);
  }
  if(gui_init(NULL) < 0){
    perror("Cannot initialize the display\n");
    exit(1);
  }
  uc_y = 30 + 40*((cnf_nr + 1)/2);
  gui_window(&uc,&ucGC,0,400,uc_y,200,200,4,0,"uni_config");
  for(i = 0; i<cnf_nr ; i++) {
    but_x = 10+200*(i%2);
    but_y = 20+40*(i/2);
    gui_window(&(uc_but[i]),&(uc_butGC[i]),&uc,100,30,but_x,but_y,1,2,"");
  }
  gui_display(&uc);
  for(i = 0;i<cnf_nr;i++) gui_display(&(uc_but[i]));
  uc_y = 40 + 40*((MAX_START + 1)/2);
  gui_window(&st,&stGC,0,400,uc_y,200,200,4,0,"  ");
  for(i = 0; i<MAX_START ; i++) {
    but_x = 10+200*(i%2);
    but_y = 30+40*(i/2);
    gui_window(&(st_but[i]),&(st_butGC[i]),&st,100,30,but_x,but_y,1,2,"");
  }
}

update_uni_config()
{
  int i;

  if(report.xkey.window == uc) update_uc();
  else if(report.xkey.window == st) update_st();
  else {
    for(i = 0;i<cnf_nr;i++){
      if(report.xkey.window == uc_but[i]) {
	update_uc_but(i);
	return(0);
      }
    }
    for(i = 0;i<n_but;i++){
      if(report.xkey.window == st_but[i]) {
	update_st_but(i);
	return(0);
      }
    }
  }
}

update_uc()
{
}

update_uc_but(but)
int but;
{
  switch(report.type){
  case Expose:
    gui_text(&(uc_but[but]),&(uc_butGC[but]),5,20,cnf_but[but].txt);
    break;
  case ButtonPress:
    gui_border(&(uc_but[but]),2);
    break;
  case ButtonRelease:
    gui_border(&(uc_but[but]),1);
    cnf_but[but].action();
    break;
  }
}

update_st()
{
  switch(report.type){
  case Expose:
    gui_text(&st,&stGC,10,20,gen_txt);
    break;
  }
}

update_st_but(but)
int but;
{
  int i;
  char tmp[100];

  switch(report.type){
  case Expose:
    gui_text(&(st_but[but]),&(st_butGC[but]),5,20,bt[but].txt);
    break;
  case ButtonPress:
    gui_border(&(st_but[but]),2);
    break;
  case ButtonRelease:
    gui_border(&(st_but[but]),1);
    bt[but].action(but);
    break;
  }
}

restore_1()
{
  int i;

  ex_flag = 1;
  for(i = 0;i<n_but;i++) gui_hide(&(st_but[i]));
  gui_hide(&st);
  gui_display(&uc);
}

check_disp2()
{
  int i;

  gui_hide(&uc);
  strcpy(bt[n_but].txt,"exit");
  bt[n_but].action = restore_1;
  n_but+=1;
  for(i = 0;i<n_but;i++) gui_display(&(st_but[i]));
  gui_display(&st);
  ex_flag = 0;
  while(ex_flag == 0){
    gui_next(&report);
    update_uni_config();
  }
}

