/*
 *   $USERUNIDAQ/analyzer/analyzer.d/uni_rtns.c
 */
/*---------------------------------------------------------------*/
/*                                                               */
/*      Adrienne A. Stipe                                        */
/*      Superconducting Super Collider Laboratory                */
/*      Aug. 24, 1993                                            */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, A. Stipe SSCL    *
*                                                                       *
************************************************************************/

#include <stdio.h>
#define BL_LEN 1024 /* size of  block for reads */

#define byte      unsigned char
#define errorVal  -1
#define successVal 0
#define  print_level 0

/*
int uni_open_file( char *fileNameP);      
int uni_close_file( );
int uni_next_event( unsigned char *bufferP, int bufferLen );
*/


/* opens *fileNameP in read mode. If successful, 0 is returned (and
   fileP points to a valid value. If unsuccessful, a non-zero value
   is returned. 
*/
static FILE *fileP;
   
int uni_open_file( fileNameP )
char *fileNameP;
   {
   fileP = fopen( fileNameP, "r" );
   if ( fileP == NULL )
      return( errorVal );
   else
      return( successVal );     
   }



/* closes the file associated with the file pointer fileP. Returns 0
   if successful, non-zero otherwise.
*/
      
int uni_close_file( )
   {
   return( fclose( fileP ) );
   } 


/* reads the next event from the file. Each event includes an event length
   and an event of size eventLength. So the total length for each event
   is eventLength + 1. The caller must provide a file pointer to an already
   opened file, a pointer to a buffer and a buffer length. This routine
   will copy the next event in the file into this buffer. An error is 
   returned if the buffer is not long enough. Errors will also be returned
   for read errors. Error values are non-zero; success return code is 0. 
*/

int uni_next_event(  data,max_length,event_length )
int *data;
int  *event_length;
int max_length;

   {
  static int block_buf[BL_LEN];
  static int first=1;
	static int bp;
	static int status;
	int pos;
	
  if( first ==1) { first=0;  
         status = read_block(block_buf);
         if(status == 0) { /* no bytes read */
             first = 1; 
         		 return (1); /* EOF  */
         }		
  		   bp =0;
  }
  
  
  pos=0;
  if (bp >= status) {
         status = read_block(block_buf) ;
         bp =0;
  }
  if(status == 0) { /* no bytes read */
             first=1;
         		 return (1) ;/* EOF  */
  }	
  
  	
  *event_length = block_buf[bp];
  if(*event_length > max_length) {
  		printf(" max_length too small in read_event \n");
  		/* the calling program should call flush_buf 
  			  and then get_buf with a new larger value of max_length 
  		*/
  			  
  		return (2);
  		}
  pos=0;
  while(pos < *event_length || *event_length == 0 ) {		
     if (bp >= status) {
         status = read_block(block_buf) ;
         bp =0;
     }
         if(status == 0) { /* no bytes read */
             first=1;
         		 return (1) ;/* EOF  */
         }		 
         if (print_level >= 30) printf(" %d %d\n",bp,block_buf[bp]);

			   if(*event_length != 0) {
  	     	data[pos]=block_buf[bp];
  	      pos++; 
  	     }
  	    else { 
  	       if ( print_level >= 10) 
  	          printf(" event_length = 0 at bp=%d\n",bp);
  	     } 
  	     bp++;
  }
return (0);
}

   


test_main()
   {
   FILE         *fileP;
   int           res;
   byte          buffer [ 20 ];
   unsigned long eventLen;
   
   /* open the data file */   
   if ( uni_open_file( "junk.dat", &fileP ) != successVal )
      {
      printf( "open error\n" );
      return;
      }
   printf( "open succeeded\n" );

   /* loop getting events */   
   res = uni_next_event( fileP, &buffer[ 0 ], sizeof( buffer ) ); 
   while( res == successVal )
      {
      /* buffer should be full -- let's print it out */
      eventLen = *( ( unsigned long * )buffer );
      printf( "length =  %d and data = ", eventLen );
      fwrite( &buffer[4], eventLen, 1, stdout );
      printf( "\n" );          
      
      /* get another... */ 
      res = uni_next_event( fileP, buffer, sizeof( buffer ) ); 
      }
        

   /* close the data file */ 
   if ( uni_close_file( fileP ) != successVal )
      {
      printf( "close error\n" );
      return;
      }
   printf( "close succeeded\n" );    
   }


int read_block(block_buf)
int *block_buf;
{ 
  int status;

  if (print_level >= 10) printf("Inside READ_BLOCK\n");
    	
  status =fread( (char *)block_buf, sizeof(int),BL_LEN, fileP) ;
  return (status);  

}

/* fortran interface */

int uni_next_event_( bufferP, bufferLen , event_length)
byte *bufferP;
int   *bufferLen;
int *event_length;
{
return uni_next_event( bufferP, bufferLen ,event_length);
}


int uni_open_file_( fileNameP )
char *fileNameP;
{
return  uni_open_file( fileNameP);
}


int uni_close_file_( )
{
return  uni_close_file();
}
