/*---------------------------------------------------------------*/
/*                                                               */
/*      Cengiz ERBAS                                             */
/*      Superconducting Super Collider Laboratory                */
/*      July 15, 1992                                            */
/*          collector.c                                          */
/*                                                               */
/*---------------------------------------------------------------*/
/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#ifdef VXWORKS
#include <msgQLib.h>
#include <stdioLib.h>
#include <time.h>
#define ERANGE     34   /* ERANGE and ETIMEDOUT hard-coded from HP errno.h */
#define ETIMEDOUT 238
#else
#include <stdio.h>
#include <errno.h>
#ifdef SGI
#define _BSD_SIGNALS
#endif
#include <signal.h>
#include <sys/time.h>
#endif

#include <string.h>

#include "collector.h"
#include "template.h"

#define HEADER_SIZE 6

int Process_Event();
int Check_Event_Usr();

/*
 *   For buffer handling
 */
#include "nova.h"
#define NOVA_HOSTNAME_MAX  32    /*  extracted from novanet.h   */
#define EVENT_SIGNIFICANCE 50
#define ENVIR_SIGNIFICANCE 70
#define PRI_COLLECTOR      -1 
#define DEFAULT_NAME       "/tmp/nova"

/*
 * Static variables for the buffer handling
 */
struct NOVAD *b_nd;    /* pointer to nova descriptor                   */
int *b_base;           /* nova buffer pointer                          */
int b_length;          /* total length (long words)                    */
int *b_record_header;  /* pointer to header of the current record      */
int *b_record_data;    /* pointer to data area of the current record   */
int b_record_length;   /* current record length                        */
int b_max_sig;         /* maximum significancy in this nova buffer     */
int b_flush_request;   /* buffer flush request                         */
/*
 * Static variables for the record handling
 */
int b_event_length;    /* return value area for the current event size */
int b_event_ID;        /* return value area for the current event ID   */
int b_event_sig;       /* return value area for the current event sig. */
int max_buf_len;       /* the declared max user data words             */

int event_semid, fake_sem; 

int local_runnr, local_eventnr, local_buffnr;

void Event_Sig_Handler_Usr();
void Process_Idle();

extern char *get_name_from_args();
extern char ownpname[];
#ifndef VXWORKS
extern long send_err_mess();
#endif

int  initial_event,initial_sec;

/****************************************************************
 * Main - main procedure of the collector
 *
 * main(argc, argv) is the main procedure of the collector. It  
 * calls the template with the addresses of the Any_Event() and
 * Process_Event() procedures.                                  
 *****************************************************************/

#ifdef VXWORKS
collector(argv1,argv2,argv3,argv4,argv5,argv6,argv7,
          argv8,argv9,argv10) 
char *argv1,*argv2,*argv3,*argv4,*argv5,*argv6,*argv7,*argv8,*argv9,*argv10;
{
  char *argv[] = {NULL,argv1,argv2,argv3,argv4,argv5,argv6,argv7,argv8,
		    argv9,argv10}; 
  int argc = 11;
#else
void main (argc, argv)
int argc;
char *argv[];
{
#endif
  char nova_name[NV_MAXNAME + 1];
  char sec_name[NV_MAXNAME + NOVA_HOSTNAME_MAX + 1];

  int i;
  short stpe;

#ifdef VXWORKS
  argv[0] = "collector";
#endif

  Create_Sem(&event_semid);

  if(Template_Init(get_name_from_args(argc,argv)) < 0){
    printf("Process of name %s already exists\n",
	   get_name_from_args(argc,argv));
    printf("Please use -n to set a different name\n");
    exit(0);
  }

  /*
   *   Open nova buffer
   */

  UNI_nova_parms(DEFAULT_NAME, PRI_COLLECTOR, nova_name, sec_name);
  if ((int)(b_nd = (struct NOVAD *)UNI_nova_open(nova_name, PRI_COLLECTOR, 1))
                 == -1) {
    printf("collector: cannot open %s", nova_name);
    Send_Message ( ownpname, "EXIT" );
  }
  b_base = (int *)0;

  /*
   *   CT modification, make all the user level variables critical
   */
  for(i = 0;i< var_number_usr; i++){
    Crit_Var(var_dict_usr[i].name);
    /*** read these variables back as well            ***/
    read_value_from_sp(ownpname,var_dict_usr[i].name,&stpe,
		       (char *)(var_dict_usr[i].addr));
  }
  /*** reset runstate to a default, idle, value.    ***/
  runstate = END_STATE;
  set_value_in_sp("runstate",&runstate);

  user_init(&max_buf_len);

  Template_Main((int (*)()) Process_Event, NULL, 
		(void (*)()) Process_Idle); 
}


/******************************************************************
 * Process_Event - calls the corresponding procedure of an event
 *
 * Process_Event() calls the corresponding procedure            
 * for a given event.  
 ******************************************************************/

int Process_Event () {
  int *record_data,*record_header;
  int record_length,event_length,event_ID;

  if (runstate == RESUME_STATE) {
    /* updata run nummber */
    if(runnr != local_runnr){
      runnr = local_runnr;
      set_value_in_sp("runnr",&runnr);
    }

    /* allocate record for one event */
    status = allocate_record();
    if (status==1) return 1;

    /* execute user routine for one buffer */
    b_event_length = 0;
    b_event_ID     = 0;
    status = user_event(b_record_data,&(b_record_length),
			&(b_event_length),&(b_event_ID),&max_buf_len,&mode);
    /* check status
     *    0:         Normal return
     *    -1:        Error
     *    1:         Timed out
     *    ETIMEDOUT: Timed out
     *    ERANGE:    Allocated area is too small
     */
    if (status==ERANGE) {
      flush_buffer_now();
      return 0;
    }
    if (status)         {
      if (b_flush_request) flush_buffer_now();
      return status;
    }

    /* save the record */
    if (b_event_ID==EVENT_RECORD) {
      eventnr++;
      local_eventnr++;
      save_record(local_runnr,eventnr,mode,0,EVENT_SIGNIFICANCE);
    }
    else {
      save_record(local_runnr,runtype,mode,0,ENVIR_SIGNIFICANCE);
    }
/*
 *     If flush_all is set, then force all events to be in a single buffer.
 */
    if (flush_all)   b_flush_request = 1;

    if (b_flush_request) flush_buffer_now();

    /*** storing event in status path ***/
    if((eventnr&0x0A)==0x0A)      {
      set_value_in_sp("eventnr",&eventnr);
      set_value_in_sp("buffnr",&buffnr);
    }
    
    if((eventnr >= maxnevents) && (maxnevents != 0)){
      Pause(" ","Collector autoPAUSEd; maxnevents reached.");
#ifndef VXWORKS
      send_err_mess(0,"Collector autoPAUSEd; maxnevents reached.");
#endif
    }
    return(TRUE);
  }
  else if(runstate == END_STATE ) {
    local_runnr = runnr;
    idle(1);
  }
  else {
    runnr = local_runnr;
    idle(1);
  }
}


/***************************************************************
 * Begin - processes a begin command
 *
 * Begin(header, arg_string) processes a begin command.         
 * header: the name of the process which sends the begin cmnd,  
 * arg_string: the arguments of the begin command.   
 ***************************************************************/

int Begin (header, arg_string) char *header, *arg_string; {
  char run_number_string[100], comment_string[100];

  
  /*
   *   Check and correct the current state
   */
  if ((runstate == BEGIN_STATE) || (runstate == RESUME_STATE)) {
    Pause("","");
    End("","");
  }
  else if (runstate == PAUSE_STATE) {
    End("","");
  }

  Parse(arg_string, run_number_string, comment_string);

  /* Set run number */
  local_runnr = runnr;
  if(strlen(run_number_string) > 0) {
    if (*run_number_string == '+') local_runnr += 1;
    else                           local_runnr = atoi(run_number_string);
    runnr = local_runnr;
  }
  eventnr = 0;
  local_eventnr = 0;
  local_buffnr = 0;
  buffnr = 0;

  user_begin(&runtype,&max_buf_len);

  /*
   *   Generate a Begin_Run record.
   */
  generate_control_record(&initial_sec,
			  comment_string,BEGINRUN_RECORD,"begin run");
  starttime   = initial_sec;
  endtime     = 0;
  runstate = BEGIN_STATE;

  /*** put variables to the status path ***/
  set_all_values_in_sp();
  
#ifndef VXWORKS
	send_err_mess(0,"Collector BEGIN command executed.");
#endif

  return TRUE;
}




/*************************************************************
 * Pause - processes a pause command
 * 
 * Pause(header, arg_string) processes a pause command.         
 * header: the name of the process which sends the pause cmnd,  
 * arg_string: the arguments of the end command.              
 *************************************************************/

int Pause (header, comment_string) char *header, *comment_string; {

  int t;

  /* allow PAUSE even if in PAUSE state */
  if ( (runstate == END_STATE)) { 
    printf("Invalid state transition...\n");
#ifndef VXWORKS
    send_err_mess(0,"Collector: Illegal PAUSE command.");
#endif
    return TRUE;
  }
  
  user_pause(&mode);
  
  /*
   *   Generate a Pause_Run record and flush.
   */
  generate_control_record(&t,
			  comment_string,PAUSERUN_RECORD,"pause run");
  runstate = PAUSE_STATE;

  printf("# seconds = %d\n", t - initial_sec);
  printf(" Total this run # events = %d; this resume %d; buffers used %d\n",
	 eventnr, local_eventnr, local_buffnr);
  printf("event rate = %f\n",
	 (float)(local_eventnr)/(float)(t - initial_sec));
  
  /*** put variables to the status path ***/
  set_all_values_in_sp();
  
#ifndef VXWORKS
	send_err_mess(0,"Collector PAUSEd.");
#endif

  return TRUE;
}

/*************************************************************
 * Resume - processes a resume command
 * 
 * Resume(header, arg_string) processes a resume command.         
 * header: the name of the process which sends the resume cmnd,  
 * arg_string: the arguments of the resume command.              
 *************************************************************/

int Resume (header, comment_string) char *header, *comment_string; {

  if (runstate == END_STATE) { 
    printf("Invalid state transition...\n");
#ifndef VXWORKS
    send_err_mess(0,"Collector: Invalid to RESUME.");
#endif
    return TRUE;
  }
 
  user_resume(&mode);

  generate_control_record(&initial_sec,
			  comment_string,RESUMERUN_RECORD,"resume run");
  runstate = RESUME_STATE;

  local_eventnr =0;
  
  /*** put variables to the status path ***/
  set_all_values_in_sp();
  
#ifndef VXWORKS
  send_err_mess(0,"Collector: Run RESUMEd.");
#endif

  return TRUE;
}



/*************************************************************
 * End - processes an end command
 * 
 * End(header, arg_string) processes a end command.         
 * header: the name of the process which sends the end cmnd,  
 * arg_string: the arguments of the end command.              
 *************************************************************/

int End (header, comment_string) char *header, *comment_string; {

#ifndef VXWORKS
  char localchar[80];
#endif

  /*
   *   Check and correct the current state.
   */
  if (runstate == END_STATE) {
    return TRUE;
  }
  else if ((runstate == BEGIN_STATE) || (runstate == RESUME_STATE)) {
    Pause("","");
  }

  user_end(&runtype);

  /*
   *   Generate an End_Run record and flush.
   */
  generate_control_record(&endtime,
			  comment_string,ENDRUN_RECORD,"end run");
  runstate = END_STATE;

  /*** put variables to the status path ***/
  set_all_values_in_sp();

#ifndef VXWORKS
  sprintf (localchar, "Collector: END run %d issued.", local_runnr);
  send_err_mess(0, localchar);
  sprintf (localchar, "Run %d, events %d, buffers %d", local_runnr,
	   local_eventnr, local_buffnr);
  send_err_mess(0, localchar);
#endif

  return TRUE;
}


/********************************************************************
 * Exit - processes an exit command
 *
 * Exit(header, arg_string) processes an exit command.
 * header:     the name of the process which sends the exit command,
 * arg_string: the arguments of the exit command.
 *********************************************************************/

int ExitC (header, arg_string) char *header, *arg_string; {
  End("","");
  if (Delete_Sem(event_semid) == ERROR)
    if (debug_level >= 10) printf("Error in Delete_Sem()\n"); 
  
  user_exit();

  /* close nova buffer manager */
  UNI_nova_close(b_nd);
  
  return(Exit("collector", " "));
}


void Process_Idle() {
  return;
}

/*
 *  Generate control record
 */ 
generate_control_record(t,comment,ID,name) int *t,ID; char *comment,*name; {
#ifndef VXWORKS
  struct timeval tp;
  struct timezone tzp;
#else
  time_t prestime;
#endif

  char *ltime;
  char *b_record_char;
  int  len,i;
  char temp[100];
  
  flush_buffer_now();
  while(allocate_record()==1);
  
  /*  Time stamp */
#ifndef VXWORKS
  gettimeofday(&tp, &tzp);
  *t = tp.tv_sec;
  ltime = ctime((time_t *)&(tp.tv_sec));
#else
  time(&prestime);
  *t = prestime;
  ltime = ctime(&prestime);
#endif
  strcpy(b_record_data,ltime);
  len = strlen(ltime);
  b_record_char = (char *)b_record_data;
  for(i=0;i<4;i++) (b_record_char)[len++] = 0;
  while(len&3)  (b_record_char)[len++] = 0;
  b_event_length = len/4;
  
  /* Copy in the comments (if any)       */
  if (strlen(comment) > 0) {
    strcpy(&((b_record_data)[b_event_length]), comment);
    b_event_length += (strlen(comment)+1)/4 + 1;
  }
  else {
    strcpy(temp, "No comment.");
    strcpy(&((b_record_data)[b_event_length]), temp);
    b_event_length += (strlen(temp)+1)/4 + 1;
  }

  b_event_ID = ID;
  save_record(local_runnr,runtype,mode,0,ENVIR_SIGNIFICANCE);
  flush_buffer_now();

  printf("Collector Run %d -- %s; \n",local_runnr,name);
  return;

}

/************************************************************************
 * flush_buffer_now()
 *  close the current nova buffer and forward it. 
 ************************************************************************/

int flush_buffer_now() {

  if ((int)(b_base)!=0) {
    *(b_record_header)++ = 0;

    if (debug_level >= 10) printf("nova put  base=%8X length=%d\n",
            (int)(b_base),(int)(b_record_header-b_base));
  
    UNI_nova_put(b_nd, b_base,(int)(b_record_header-b_base)*4,b_max_sig);
    b_base   = (int *)0;
    b_length = 0;
  }
  b_flush_request = 0;
  set_value_in_sp("eventnr",&eventnr);
  return 0;
}

/************************************************************************
 * flush_buffer(header, comment_string)
 *  request to flush buffer
 ************************************************************************/

int flush_buffer(header,comment_string) char *header, *comment_string; {
  printf("FLUSH request\n");
  b_flush_request = 1;
  return TRUE;
}

/*********************
 * allocate_record() *
 *********************/

int allocate_record() {
  int *header;


  /* return the address for the next writeable location */
  if ((int)b_base == 0) {
    if (UNI_nova_get(b_nd, &(b_base), &(b_length))==1) return 1;
    b_record_header = b_base;
    b_max_sig = 0;
    b_length >>= 2;
    local_buffnr += 1;
    buffnr = local_buffnr;
    if (debug_level >= 10) printf("nova get  base=%8X\n",(int)(b_base));
  }

  header = b_record_header;
  b_record_data = &(header[HEADER_SIZE]);
  header[0] = 0;
  header[1] = 0;
  header[5] = 0;
  b_record_length = b_length - HEADER_SIZE - (int)((b_record_header-b_base));


  return 0;
}

/*****************
 * save_record() *
 *****************/

int save_record(run_no, evt_no, hdr_mode, hdr_reserve, significance) 
int run_no, evt_no, hdr_mode, hdr_reserve, significance; 
{
  int *header;
  
  /* check the event length for consistency */
  if (b_event_length + HEADER_SIZE > 
      b_length - (int)(b_record_header-b_base)) return -1;
  
  header = b_record_header;
  header[0] = b_event_length + HEADER_SIZE;
  header[1] = b_event_ID;
  header[2] = run_no;
  header[3] = evt_no;
  header[4] = hdr_mode;
  header[5] = hdr_reserve;
  if(significance > b_max_sig) b_max_sig=significance;
  
  /* advance pointer to next record by right number of words*/
  b_record_header += (b_event_length + HEADER_SIZE);
  
  return 0;
}

/*
 *     Fortran entries, for those so inclined.
 */

pause_(header,arg)
char *header,*arg;
{
return	Pause(header,arg);
}
resume_(header,arg)
char *header,*arg;
{
return	Resume(header,arg);
}
process_event_()
{
return	Process_Event();
}

int mset(header,comment_string) char*header, *comment_string; {
/*
 * mset(header,comment_string)
 *  command exists solely to allow the setting of many variables
 *  in one command string.
 */
  return TRUE;
}
