# browser for UNIDAQ processes
#==========================================
#
#  Original Eric Babe, University of Michigan, March 31, 1995.
#  Modified 4/6/95   BB   Diddle a few positions, colors, ...
#
#==========================================

# -----------  PHYSICAL PARAMETERS  -------------
# \/------ Maximum Machines Displayed on One Screen
set machs_displayed 5
# -----------  GLOBALS  -----------
set old_left_mach 1
set old_max_frame 0
set procs_per_mach {0}
set box_width 10
set max_frame 1
# \/------ A List of All Machines
set machines {0}
set machines_active {0}
set no_machines 0
# \/------ This is the quantity tied to the scrollbar.
set left_proc 1
set no_procs 0
set old_left_sub_proc 1
set old_left_proc 1
set left_mach 1
set registry 0
set registry_size 0

# ----------  SCROLLBAR/TITLE WIDGET INSTANTIATION  ----------
scrollbar .sbhsb -orient horizontal -command {scrollbar_to_screen} -relief sunken
pack .sbhsb -side bottom -fill x
label .lbuni -text "        UNIDAQ        " -relief sunken
pack .lbuni -side top -fill x

# -----------  UPDATE  ------------
# This is the code bound to the 'Update' button.
# It's job is to destroy all the widgets and then resurge them.
# start_data is called both during the startup sequence and 
# during an update, with no special flags (it's update's job 
# to make sure that start_data has a clean environment
# to work with [ie an environment equivalent to initial config]).
# ---------------------------------
proc update noargs {
    global left_mach no_machines old_left_mach machs_displayed old_max_frame box_width
    global procs_per_mach max_frame registry registry_size

    # ------- Kill All Widgets
    set loop $max_frame
    while {$loop >= 1} {
	destroy .frame_$loop 
	set loop [expr $loop-1]
    }
    init_globals 0
    # ------- \/...Includes update and exit buttons
    destroy .bottom_frame
    start_data 0
    draw_screen 0
}

# -----------  INITIALIZE GLOBALS  ------------
proc init_globals noargs {
    global no_machines left_mach old_left_mach machs_displayed old_max_frame procs_per_mach
    global box_width max_frame machines registry registry_size
    set no_machines 0
    set left_mach 1
    set old_left_mach 1
    set machs_displayed 5
    set old_max_frame 0
    set procs_per_mach {0}
    set box_width 10
    set max_frame 1
    set machines {0}                  
}

# ------------  INSTANTIATE WIDGETS  -----------
# Purpose:  Read data from UNIwish; Put data into widgets.
# Does not pack any widgets.
# Maintains registry of open process boxes (closing and re-opening all
# to update variable data); see comments on new_registry.
# A general outline of this function:
#   (1) Instantiate All Static Buttons (update, etc.)
#   (2) Get All Processes from UNIwish 'status_num_var' and 
#       'status_var_info'
#   (3) Loop through all processes
#       (a) If machine associated with process is not new, add
#           process to that machine.
#       (b) Otherwise: add new machine, new process.
#       NOTE: (a) and (b) have similar characteristics:
#           (i)   A check is made to see if process is registered
#                 (ie expanded).  If yes, process is added to
#                 new registry (see article 4).
#           (ii)  Machine is entered as default ON (==1) in 
#                 machines_active list.  This list is used in
#                 draw_screen to determine the machines which
#                 draw their procedures.  A procedure is bound
#                 to each machine button which toggles its entry
#                 in the machines_active list.  OFF==0.
#           (iii) Color of box is determined by process status
#                 (Green3==Alive, Red==Dead, Yellow==Busy)
#                 An asterisk is inserted in the title of Dead procs.
#                 An exclamation is inserted in the title of Busy procs.
#           (iv)  Command to expand proc is bound to proc buttons.
#   (4) All processes in the new_registry are closed and then
#       re-registered/re-expanded as though they had never been registered.
#       This effectively updates the variable data in the child
#       boxes. 
# ----------------------------------------------
proc start_data noargs {
    global procs_per_mach box_width no_variables status_list no_machines machines
    global registry registry_size machines_active
    # ------ New Registry Locals
    # About new_registry_xxxxx ........
    # As part of the UPDATE sequence, the registry (list of open process windows)
    # is re-evaluated during start_data.
    # Elements of the registry which are still ALIVE and extant are added
    # to the new_registry.  At the end of start_data, the old registry is
    # assassinated, and each element of the new_registry is re-registered
    # (sic added to the regular registry), and then the new_registry
    # info is discarded.
    # ------
    set new_registry_machine {0}
    set new_registry_process {0}
    set new_registry_size 0

    # ------ UPDATE and EXIT BUTTONS
    label .bottom_frame -height 2
    button .bottom_frame.update -relief raised -text Update -width $box_width -height 2 -foreground white -background red -command {update 0}
    button .bottom_frame.exit -relief raised -text Exit -width $box_width -height 2 -foreground white -background red -command {graceful_exit 0}

    # ------ get total number of processes
    set no_status_entries [status_num_var]

    # ------ loop thru all processes as obtained praetextum
    set loop 0
    while {$loop < $no_status_entries} {
	# ------ Search list of machines
	set status_entry [status_var_info $loop]
	set mloop $no_machines
	set add_new_machine 1
	while {$mloop >= 1} {
	    if {[lindex $machines $mloop]==[lindex $status_entry 1] && [lindex $machines $mloop]!=""} {
		# ------ Found a machine that has already been found
		# ------ inc procs per machine
		set upt [lindex $procs_per_mach $mloop]
		incr upt
		set procs_per_mach [lreplace $procs_per_mach $mloop $mloop $upt]
		set gtemp [lindex $procs_per_mach $mloop]

		# ------ Button for process
		set my_machine_name  [lindex $machines $mloop]
		set my_process_name  [lindex $status_entry 0]
		# ------ Set Command that gets executed when process 
		# ------ button is pressed
		set my_command       {expand_process}
		append my_command " " $my_machine_name " " $my_process_name
		# ------ If the machine/process pair is on old registry, 
		# ------ add to new_registry
		if {[lsearch -exact $registry .$my_machine_name$my_process_name] != -1} {
		    lappend new_registry_machine $my_machine_name
		    lappend new_registry_process $my_process_name
		    incr new_registry_size
		}
	
		# ---------- get button color
		set color purple
		set proctitle [lindex $status_entry 0]
		if {[lindex $status_entry 2]=="ALIVE"} {
		    set color green3
		}
		if {[lindex $status_entry 2]=="DEAD"} {
		    set color red
		    set my_command ""
		    append proctitle " (DEAD) "
		}
		if {[lindex $status_entry 2]=="BUSY"} {
		    set color yellow
		    set my_command ""
		    append proctitle " (BUSY) "
		}

 		button .frame_$mloop.varlbox_($mloop,$gtemp) -relief raised -text $proctitle  -width $box_width -height 2 -background $color -foreground white -command $my_command 
		# ------ Frame for each vertical bar
		frame .frame_$mloop.mfr.vfr_($mloop,$gtemp) -height 3 -width $box_width
		# ------ Vertical bar
		label .frame_$mloop.mfr.vfr_($mloop,$gtemp).vfr -relief raised -height 3 -width 0
		# ------ Make sure don't add new machine
		set add_new_machine 0
	    }
	    set mloop [expr $mloop-1]
	}

	# ------ Add new machine
	if {$add_new_machine==1} {
	    set status_entry [status_var_info $loop]
	    # ------ Initialize Machine Info
	    lappend procs_per_mach 1
	    lappend machines_active 1
	    lappend machines [lindex $status_entry 1]
	    
	    incr no_machines
	    set mloop $no_machines

	    # ------ New Machine Instantiation
	    ## Frame to enclose a machine and all processes
	    frame .frame_$mloop -relief raised -height 10  -width $box_width
	    # 
	    # ------ Button for Machine -------
	    set mach_command {toggle_machine } 
	    append mach_command $no_machines

	    button .frame_$mloop.procbox -height 5 -width $box_width -text [lindex $status_entry 1] -command $mach_command
	    #	
	    # ------ Frame for first vertical bar and horizontal bar -------
	    frame .frame_$mloop.mfr -relief raised -height 3 -width $box_width
	    #	
	    # ------ Button for horizontal bar -------
	    label .frame_$mloop.hfr -pady 0 -relief raised -height 0 -width $box_width
	    #	
	    # ------ Button for first vertical bar -------
	    label .frame_$mloop.vfr -relief raised -height 3 -width 0
	    
	    # ---------- setup button command
	    set my_machine_name  [lindex $machines $mloop]
	    set my_process_name  [lindex $status_entry 0]
	    set my_command       {expand_process} 
	    append my_command " " $my_machine_name " " $my_process_name
	    # ------ If the machine/process pair is on old registry, 
	    # ------ add to new_registry
	    if {[lsearch -exact $registry .$my_machine_name$my_process_name] != -1} {
		lappend new_registry_machine $my_machine_name
		lappend new_registry_process $my_process_name
		incr new_registry_size
	    }
	    # ---------- get button color
	    set color purple
	    if {[lindex $status_entry 2]=="ALIVE"} {
		set color green3
	    }
	    if {[lindex $status_entry 2]=="DEAD"} {
		set color red
		set my_command ""
		append proctitle " (DEAD) "
	    }
	    if {[lindex $status_entry 2]=="BUSY"} {
		set color yellow
		append proctitle " (BUSY) "
	    }

	    # ------ Button for process
	    button .frame_$mloop.varlbox_($mloop,1) -relief raised -text [lindex $status_entry 0] -width $box_width -height 2 -background $color -foreground white -command $my_command
	    # ------ Frame for each vertical bar
	    frame .frame_$mloop.mfr.vfr_($mloop,1) -height 3 -width $box_width
	    # ------ Vertical bar
	    label .frame_$mloop.mfr.vfr_($mloop,1).vfr -relief raised -height 3 -width 0
	}
	incr loop
    }

    ## ------ Kill Registry
    set rloop $registry_size
    while {$rloop>=1} {
	kill_process [lindex $registry $rloop]
	set rloop [expr $rloop-1]
    }

    ## ------ Re-register everything in New Registry
    set registry {0}
    set registry_size 0
    set rloop 1
    while {$rloop<=$new_registry_size} {
	expand_process [lindex $new_registry_machine $rloop] [lindex $new_registry_process $rloop] 
	incr rloop
    }
}

# -----------  scrollbar to screen  -------------------
# The scrollbar is set to scroll processes.
# Scrollbar parameters are established at the end of draw_screen.
# ---------------------------------
proc scrollbar_to_screen y {
    global left_proc no_procs
    set left_proc_was $left_proc

    set left_proc [expr $y+1]
    if {$left_proc<1} {set left_proc 1}
    if {$left_proc>$no_procs} {set left_proc $no_procs}

    if {$left_proc!=$left_proc_was} {draw_screen 0}
}

# -----------  draw screen procedure  -----------------
# Purpose:  draw/redraw the screen
# (1) Counts processes; finds first machine to be drawn;
#     finds first process of that machine to be drawn.
# (2) Calculates old data for machine and first process.
# (3) Kills as widgets that were drawn in previous call
#     to draw_screen as per (2).  
#     If no previous call, none are killed.
# (4) Packs widgets for machines that are toggled 'active'
# (5) Maintains window geometry and scrollbar.
# -------------------------------------------
proc draw_screen noargs {
    global left_mach no_machines old_left_mach machs_displayed old_max_frame box_width
    global procs_per_mach max_frame machines_active procs_per_mach left_proc old_left_proc
    global old_left_sub_proc no_procs
    set max_frame [expr $left_mach+$machs_displayed]

    # -- figure out total number of processes
    set no_procs 0
    set loop 1
    while {$loop <= $no_machines} {
	set no_procs [expr $no_procs+[lindex $procs_per_mach $loop]]
	incr loop
    }
    # -- find left_mach and left_sub_proc based on left_proc
    set proc_count 0
    set loop 1
    while {$loop <= $no_machines} {
	set previous_proc_count $proc_count
	set proc_count [expr $proc_count+[lindex $procs_per_mach $loop]]
	if {$proc_count>=$left_proc} {
	    set left_mach $loop
	    set left_sub_proc [expr $left_proc-$previous_proc_count]
	    if {$left_sub_proc<1} {
		set left_sub_proc 1
	    }
	    # \/...break from loop
	    set loop [expr $no_machines+1]
	}
	incr loop
    }

    # -- find old_left_sub_proc (which isn't really that old after all)
    set proc_count 0
    set loop 1
    while {$loop <= $no_machines} {
	set previous_proc_count $proc_count
	set proc_count [expr $proc_count+[lindex $procs_per_mach $loop]]
	if {$proc_count>=$old_left_proc} {
	    set old_left_mach $loop
	    set old_left_sub_proc [expr $old_left_proc-$previous_proc_count]
	    if {$old_left_sub_proc<1} {
		set old_left_sub_proc 1
	    }
	    # \/...break from loop
	    set loop [expr $no_machines+1]
	}
	incr loop
    }

    if {$max_frame > $no_machines} {
	set max_frame $no_machines
    }

    #
    # Unmanage and discard all screen listboxes, canvases
    #
    if {$old_left_mach>0} {
	set loop $old_left_mach
	while {$loop <= $old_max_frame} {
	    pack forget .frame_$loop
	    pack forget .frame_$loop.vfr
	    pack forget .frame_$loop.hfr
	    pack forget .frame_$loop.mfr

	    set gloop 1
	    if {$old_left_mach==$loop} {
		set gloop $old_left_sub_proc
	    }
	    while {$gloop <= [lindex $procs_per_mach $loop]} {
		pack forget .frame_$loop.mfr.vfr_($loop,$gloop)
		pack forget .frame_$loop.mfr.vfr_($loop,$gloop).vfr
		incr gloop
	    }

	    set gloop 1
	    if {$old_left_mach==$loop} {
		set gloop $old_left_sub_proc
	    }
	    while {$gloop <= [lindex $procs_per_mach $loop]} {
		pack forget .frame_$loop.varlbox_($loop,$gloop) 	    
		incr gloop
	    }

	    incr loop
	}
	pack forget .bottom_frame
#
#       Begin repack now
#
	pack .bottom_frame -side bottom -fill x
	pack .bottom_frame.update -side left
	pack .bottom_frame.exit -side left
    }

    #
    # Repack them beginning with left_mach
    # 
    set loop $left_mach
    while {$loop <= $max_frame} {
	pack .frame_$loop -side left -padx 1m -pady 1m -anchor n
	pack .frame_$loop.procbox -side top -pady .1
	if {[lindex $machines_active $loop]==1} {
	    pack .frame_$loop.vfr 
	    pack .frame_$loop.hfr -fill x -padx 50 -ipady 0 -pady 0
	    pack .frame_$loop.mfr -fill x
	    set gloop 1
	    if {$left_mach==$loop} {
		set gloop $left_sub_proc
	    }
	    while {$gloop <= [lindex $procs_per_mach $loop]} {
		pack .frame_$loop.mfr.vfr_($loop,$gloop) -side left -padx [expr 5.8*$box_width]
		pack .frame_$loop.mfr.vfr_($loop,$gloop).vfr
		incr gloop
	    }

	    set gloop 1
	    if {$left_mach==$loop} {
		set gloop $left_sub_proc
	    }
	    while {$gloop <= [lindex $procs_per_mach $loop]} {
		pack .frame_$loop.varlbox_($loop,$gloop) -side left -padx 10 -ipadx 3 -anchor w	    
		incr gloop
	    }
	}
	incr loop
    }

    wm geometry . 764x400
    #
    # tell scrollbar whats up
    #
    .sbhsb set [expr $no_procs] [expr 1] [expr $left_proc-1] [expr $left_proc-1]
    #
    # remember old values
    #
    set old_max_frame $max_frame
    set old_left_mach  $left_mach
    set old_left_proc $left_proc
    set old_left_sub_proc $left_sub_proc
}

# --------------  expand process into child window -----------
# (1) Gets data from proc_num_var; if fails, exits quietly.
# (2) Removes entry from registry of open processes if
#     process is already registered.
# (3) Registers process in 'registry' list.
# (4) Instantiates widget and subwidgets.
# ---------------------------------- 
proc expand_process {mach name} {
    global registry registry_size

    set no_var [proc_num_var $name]
    if {$no_var != "Error in read_value"} {
	# ------ see if child is registered; if yes, destroy it
	if {[lsearch -exact $registry .$mach$name] != -1} {
	    kill_process .$mach$name
	}
	# ------ register child
	lappend registry .$mach$name
	incr registry_size
	
	# ------ instantiate widget
	toplevel .$mach$name
	#wm geometry .$mach$name 300x300
	set mycommand kill_process
	append mycommand " " .$mach$name
	wm protocol .$mach$name WM_DELETE_WINDOW $mycommand

	# ------- set process box title
	set wintitle "$mach : $name"
	wm title .$mach$name $wintitle
	
	# ------- add child widgets to process window
	listbox .$mach$name.lbox 
	set loop 1
	while {$loop <= $no_var} {
	    .$mach$name.lbox insert end [proc_var_num $name $loop]
	    incr loop
	}
	set butncmnd kill_process
	append butncmnd " " .$mach$name
	button .$mach$name.exit -text "  Exit   " -command $butncmnd
	pack .$mach$name.lbox -anchor n
	pack .$mach$name.exit -anchor s
    }
    if {$no_var == "Error in read_value"} {
	puts "UNI routine 'proc_num_var' returned:  'Error in read_value' for"
	puts $name
	puts " "
    }
}

# ----------  kill process child box (infanticide) ----------
# (1) Destroys widget.
# (2) Removes from registry.
# ---------------------------
proc kill_process {name} {
    global registry registry_size
    destroy $name
    set registry [lreplace $registry [lsearch -exact $registry $name] [lsearch -exact $registry $name]]
    set registry_size [expr $registry_size-1]
}


# ----------  toggle machine active state  ----------------
# Changes entry in 'machines_active' list corresponding 
# to machine 'mc'.
# This procedure is bound to each machine button and is
# used to turn machines ON or OFF.  Machines that are
# OFF do not display sub-processes.
# ------------------------------------- 
proc toggle_machine mc {
    global machines_active
    set machines_active [lreplace $machines_active $mc $mc [expr 1-[lindex $machines_active $mc]]]
    draw_screen 0
}


# ----------  exits program  ---------
# Exits Program.  This routine is bound to the Exit button.
# It is modeled on the exit routine in collector.tcl
# in the conf directory.
# -------------------------
proc graceful_exit noargs {
    destroy .
}

# ==============================  MAIN  =================================
init_globals 0
start_data 0
draw_screen 0






