#!/bin/csh
#---------------------------------------------------------------#
#          Setup file for UNIDAQ environment variables.         #
#          This file is named "setup.csh" and is                #
#          located in the conf directory of the USERUNIDAQ      #
#	   system.  A link to this file by the same name is     #
#	   provided in the top level directory.                 #
#          Source this file once at login, or from your .cshrc  #
#          file.  Sourcing of the $TOPUNIDAQ/setup.csh file     #
#	   is not needed, but if done, $TOPUNIDAQ should be     #
#	   first.                                               #
#---------------------------------------------------------------#

# Setting aliases
alias @env-var-rm\
'if (${?\!:1} == 0)  setenv \!:1 "";'\
'setenv \!:1 `echo :${\!:1} | sed -e s~:\!:2~~g -e s~^:~~`;'\
'if ("${\!:1}" == "")  unsetenv \!:1;'\

alias @env-var-prepend\
'@env-var-rm \!:1 \!:2;'\
'if (${?\!:1} == 0)  setenv \!:1 "";'\
'if ("${\!:1}" != "")  setenv \!:1 :"$\!:1";'\
'setenv \!:1 "\!:2$\!:1"'

alias @env-var-append\
'@env-var-rm \!:1 \!:2;'\
'if (${?\!:1} == 0)  setenv \!:1 "";'\
'if ("${\!:1}" != "")  setenv \!:1 "$\!:1": ;'\
'setenv \!:1 "${\!:1}"\!:2'\

alias @path-rm 'set path = (`echo " $path " | sed "s~ \!:1 ~ ~g"`)'
alias @path-prepend '@path-rm \!:1; set path = (\!:1 $path)'
alias @path-append '@path-rm \!:1; set path = ($path \!:1)'

# Remove old UNIDAQ PATH and MANPATH traces, if any
if ( ${?TOPUNIDAQ} ) then
    @env-var-rm MANPATH $TOPUNIDAQ/man
    @path-rm $TOPUNIDAQ/bin
    @path-rm $TOPUNIDAQ/NOVA/src
endif

# Remove old user UNIDAQ PATH, if any
if ( ${?USERUNIDAQ} ) then
    @path-rm $USERUNIDAQ/bin
endif

# Now set the variables for this version of UNIDAQ
setenv TOPUNIDAQ          /export/home/onlsun3/igarashi/UNIDAQ/v2.3
setenv USERUNIDAQ         /export/home/onlsun3/igarashi/UNIDAQ/v2.3/user_UNIDAQ

setenv UNIDAQversion      2.3

setenv MACHINE            SUN
setenv MACFLAV            SOLARIS2
setenv CERN_ROOT          /kek/cern/pro
setenv MURMUR_CLIENT_DIR  $TOPUNIDAQ/murmur_client/current

if ( ${?TOPTCLTK} ) then
   unsetenv TOPTCLTK
endif
setenv CC                 gcc
setenv X11LIB             "-L/usr/openwin/lib -lX11"
setenv CX11FLAGS          -I/usr/openwin/include 
setenv LDFLAGS            ""
setenv F77FLAGS           ""
setenv FF                 f77
setenv SUN_EXTLIB         "-L/usr/ucblib -lsocket -lucb -lnsl -ldl -R/usr/lib:/usr/ucblib" 
setenv BVMELIB            "" 

setenv MURMUR_SERVER_ADD  134.3.30.43

# Set the PATH correctly for this UNIDAQ version
@path-prepend $TOPUNIDAQ/bin
@path-prepend $TOPUNIDAQ/NOVA/src
@path-prepend $USERUNIDAQ/bin
set dmantst=`printenv|grep MANPATH`
if ( $#dmantst > 0 ) then
  @env-var-prepend MANPATH $TOPUNIDAQ/man
else
  setenv MANPATH          $TOPUNIDAQ/man:/usr/man
endif
unset dmantst
rehash

setenv LGBK_LOC           "$USERUNIDAQ/log/log.book"
